@@ -19,17 +19,43 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
 #ifndef _VICE_JOYSTICK_H
 #define _VICE_JOYSTICK_H
 
-/* Flag: is joystick present? */
-extern int joystick_available;
+#include "kbd.h"
+
+typedef enum {
+    JOYDEV_NONE,
+    JOYDEV_NUMPAD,
+    JOYDEV_KEYSET1,
+    JOYDEV_KEYSET2,
+    JOYDEV_HW1,
+    JOYDEV_HW2
+} joystick_device_t;
+
+typedef enum {
+    KEYSET_NW,
+    KEYSET_N,
+    KEYSET_NE,
+    KEYSET_E,
+    KEYSET_SE,
+    KEYSET_S,
+    KEYSET_SW,
+    KEYSET_W,
+    KEYSET_FIRE
+} joystick_direction_t;
 
 extern void joystick_init(void);
+extern int joystick_init_resources(void);
+extern int joystick_init_cmdline_options(void);
 extern void joystick_update(void);
+extern int joystick_handle_key(kbd_code_t kcode, int pressed);
+extern const char *joystick_direction_to_string(joystick_direction_t
+                                                direction);
 
 #endif
