@@ -19,12 +19,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
 #ifndef _TUIFS_H
 
-extern char *tui_file_selector(const char *title, const char *pattern);
+extern char *tui_file_selector(const char *title, const char *directory,
+			       const char *pattern, const char *default_item,
+			       char *(*read_contents_func)(const char*));
 
 #endif
