@@ -19,7 +19,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
@@ -27,15 +28,29 @@
 #define _UI_DOS_H
 
 #include "types.h"
+#include "tuimenu.h"
 
-extern int UiInit(int *argc, char **argv);
-extern int UiInitFinish(void);
-extern void UiError(const char *format, ...);
-extern int UiJamDialog(const char *format, ...);
-extern void UiShowText(const char *title, const char *text);
-extern void UiUpdateMenus(void);
-extern void UiAutoRepeatOn(void);
-extern void UiAutoRepeatOff(void);
-extern void UiMain(ADDRESS);
+typedef enum {
+    UI_JAM_RESET, UI_JAM_HARD_RESET, UI_JAM_MONITOR, UI_JAM_DEBUG
+} ui_jam_action_t;
+
+/* ------------------------------------------------------------------------- */
+
+extern int ui_init_resources(void);
+extern int ui_init_cmdline_options(void);
+
+extern int ui_init(int *argc, char **argv);
+extern int ui_init_finish(void);
+extern void ui_error(const char *format, ...);
+extern ui_jam_action_t ui_jam_dialog(const char *format, ...);
+extern void ui_show_text(const char *title, const char *text);
+extern void ui_update_menus(void);
+extern void ui_main(char hotkey);
+extern void ui_toggle_drive_status(int state);
+extern void ui_display_drive_track(double track_number);
+extern void ui_display_drive_led(int status);
+extern void ui_set_warp_status(int status);
+extern void ui_dispatch_events(void);
+extern int ui_extend_image_dialog(void);
 
 #endif
