@@ -21,7 +21,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
@@ -31,11 +32,11 @@
 #define _KBDEF_X_H
 
 enum shift_type {
-    NO_SHIFT = 0,		/* Key is not shifted. */
-    VIRTUAL_SHIFT = 1 << 0,	/* The key needs a shift on the real machine. */
-    LEFT_SHIFT = 1 << 1,	/* Key is left shift. */
-    RIGHT_SHIFT = 1 << 2,	/* Key is right shift. */
-    ALLOW_SHIFT = 1 << 3,	/* Allow key to be shifted. */
+    NO_SHIFT =(0),		/* Key is not shifted. */
+    VIRTUAL_SHIFT =(1 << 0),	/* The key needs a shift on the real machine. */
+    LEFT_SHIFT =(1 << 1),	/* Key is left shift. */
+    RIGHT_SHIFT =(1 << 2),	/* Key is right shift. */
+    ALLOW_SHIFT =(1 << 3),	/* Allow key to be shifted. */
 };
 
 typedef struct {
@@ -45,6 +46,8 @@ typedef struct {
     enum shift_type shift;
 } keyconv;
 
+#if 0 /*********************************************************************/
+
 #ifdef VIC20
 #define ROW0	7
 #define ROW7	0
@@ -60,7 +63,7 @@ typedef struct {
 /* VIC20/C64/C128 keyboard maps. */
 
 #ifndef PET
-static keyconv keyconvmap[] =
+static keyconv default_keyconvmap[] =
 {
     { XK_Delete, ROW0, 0, ALLOW_SHIFT },	/* del */
     { XK_Return, ROW0, 1, ALLOW_SHIFT },
@@ -289,15 +292,22 @@ static keyconv pet80map[] =
     { XK_Return, 3, 4, NO_SHIFT },
     { XK_Escape, 2, 0, NO_SHIFT },	/* Escape */
 
-    { XK_Tab, 8, 0, ALLOW_SHIFT },		/* Tab -> rev/off */
-#ifdef hpXK_BackTab
-    { hpXK_BackTab, 8, 0, VIRTUAL_SHIFT },	/* Tab -> rev/off */
-#endif
-    { XK_KP_Tab, 8, 0, ALLOW_SHIFT },
-#ifdef hpXK_KP_BackTab
-    { hpXK_KP_BackTab, 8, 0, VIRTUAL_SHIFT },
-#endif
+    { XK_Alt_L, 8, 0, ALLOW_SHIFT },		/* Left Alt -> RVS */
+    { XK_Meta_L, 8, 0, ALLOW_SHIFT },		/* Left Meta -> RVS */
 
+    { XK_Control_R, 7, 6, ALLOW_SHIFT },	/* Right Ctrl -> RPT */
+    
+    { XK_Tab, 4, 0, ALLOW_SHIFT },		/* Tab -> Tab */
+#ifdef XK_ISO_Left_Tab
+    { XK_ISO_Left_Tab, 4, 0, VIRTUAL_SHIFT },   /* Left Tab -> Shift-Tab */
+#endif
+#ifdef hpXK_BackTab
+    { hpXK_BackTab, 4, 0, VIRTUAL_SHIFT },
+#endif
+#ifdef hpXK_KP_BackTab
+    { hpXK_KP_BackTab, 4, 0, VIRTUAL_SHIFT },
+#endif
+    
     { XK_KP_0, 7, 4, ALLOW_SHIFT },	/* Numpad 0 */
     { XK_KP_1, 8, 7, ALLOW_SHIFT },	/* Numpad 1 */
     { XK_KP_2, 7, 7, ALLOW_SHIFT },	/* Numpad 2 */
@@ -428,24 +438,32 @@ keyconv pet40map[] =
     { XK_Delete, 1, 7, NO_SHIFT },
 
     { XK_Tab, 9, 0, ALLOW_SHIFT },		/* Tab -> rev/off */
+    { XK_KP_Tab, 9, 0, ALLOW_SHIFT },
+#ifdef XK_ISO_Left_Tab
+    { XK_ISO_Left_Tab, 9, 0, VIRTUAL_SHIFT },   /* Shift-Tab */
+#endif
 #ifdef hpXK_BackTab
     { hpXK_BackTab, 9, 0, VIRTUAL_SHIFT },	/* Tab -> rev/off */
 #endif
-    { XK_KP_Tab, 9, 0, ALLOW_SHIFT },
 #ifdef hpXK_KP_BackTab
     { hpXK_KP_BackTab, 9, 0, VIRTUAL_SHIFT },	/* Tab -> rev/off */
 #endif
+    
+    { XK_Alt_L, 9, 0, ALLOW_SHIFT },		/* Left Alt -> rev/off */
+    { XK_Meta_L, 9, 0, ALLOW_SHIFT },		/* Left Meta -> rev/off */
 
-    { XK_Escape, 9, 4, ALLOW_SHIFT },	/* Escape -> Run/Stop */
-    { XK_Find, 0, 6, NO_SHIFT },	/* Home */
+    { XK_Escape, 9, 4, ALLOW_SHIFT },		/* Escape -> Run/Stop */
+    { XK_Control_L, 9, 4, ALLOW_SHIFT },	/* Left Ctrl -> Run/Stop */
+    
+    { XK_Find, 0, 6, NO_SHIFT },		/* Home */
 
     { XK_Shift_L, 8, 0, LEFT_SHIFT },
     { XK_Shift_R, 8, 5, RIGHT_SHIFT },
 
 #ifdef DECMAP
     { DXK_Remove, 1, 7, NO_SHIFT },	/* del (DECkeysym.h) */
-    { XK_Help, 9, 4, ALLOW_SHIFT },		/* because there is no real ESC-key
-				   on the keyboard /RH/ */
+    { XK_Help, 9, 4, ALLOW_SHIFT },	/* because there is no real ESC-key
+					   on the keyboard /RH/ */
 #endif				/* DECMAP */
 
     { XK_a, 4, 0, NO_SHIFT },
@@ -574,13 +592,16 @@ keyconv pet40map[] =
     { 0, 0, 0, 0 }
 };
 
-static keyconv *keyconvmap = pet80map;
-static int CONV_KEYS = sizeof(pet80map) / sizeof(keyconv);
-
+/* static keyconv *keyconvmap = pet80map; */
+/* static int CONV_KEYS = sizeof(pet80map) / sizeof(keyconv); */
 #endif				/* PET */
 
 #ifndef PET
-#define CONV_KEYS (sizeof(keyconvmap)/sizeof(keyconv))
+/* #define CONV_KEYS (sizeof(keyconvmap)/sizeof(keyconv)) */
 #endif
 
+#endif /*******************************************************************/
+
+static keyconv *keyconvmap = NULL;
+
 #endif /* !_KBDEF_X_H */
