@@ -19,16 +19,22 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
 #ifndef _VSYNC_UNIX_H
 #define _VSYNC_UNIX_H
 
+#include "types.h"
+
 extern void suspend_speed_eval(void);
-extern void vsync_init(void);
+extern int vsync_init_resources(void);
+extern int vsync_init_cmdline_options(void);
+extern void vsync_init(double hz, long cycles, void (*hook)(void));
 extern int do_vsync(int been_skipped);
 extern int vsync_disable_timer(void);
+extern void vsync_prevent_clk_overflow(CLOCK sub);
 
 #endif /* !_VSYNC_UNIX_H */
