@@ -21,7 +21,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
@@ -35,7 +36,8 @@
 #include "asm.h"
 #include "misc.h"
 #include "mshell.h"
-#include "mem.h"
+
+extern BYTE ram[];		/* FIXME: ugly! */
 
 /* ------------------------------------------------------------------------- */
 
@@ -370,12 +372,12 @@ struct lookup_tag lookup[] = {
     /* ff */ { "ISB",	ABSOLUTE_X, M_ABSX, M_ABSX, 7, 0 }
 };
 
-static char *modename[] = {
+char *modename[] = {
     "IMPLIED", "ACCUMULATOR", "IMMEDIATE",
 #ifdef EXTENDED_CPU
     "IMMEDIATE_WORD", "ZERO_PAGE", "ZERO_PAGE_X", "ZERO_PAGE_Y", "ABSOLUTE",
     "ABSOLUTE_X", "ABSOLUTE_Y", "INDIRECT", "INDIRECT_X", "INDIRECT_Y",
-    "INDIRECT_Z", "ABS_INDIRECT", "ABS_INDIR_X", "RELATIVE", "RELATIVE_LONG", 
+    "INDIRECT_Z", "ABS_INDIRECT", "ABS_INDIR_X", "RELATIVE", "RELATIVE_LONG",
     "ZERO_RELATIVE", "STACK_RELATIVE_Y",
 #else
     "ZERO_PAGE", "ZERO_PAGE_X", "ZERO_PAGE_Y", "ABSOLUTE", "ABSOLUTE_X",
@@ -390,8 +392,10 @@ static char *modename[] = {
 
 /* local */
 
+#if 0
+/* DS : NO LONGER USED */
 static int interpret_line(char *, ADDRESS *, int );
-static int interpret_instr(char *, ADDRESS , int );
+int interpret_instr(char *, ADDRESS , int );
 int find_instr(char *);
 static int parse_arguments(ADDRESS , int , char *, BYTE *, int *, int );
 
@@ -496,7 +500,7 @@ static int interpret_line(char *line, ADDRESS *addr, int mode)
     ADDRESS adr = *addr;
 
     do {
-	while (*line && isspace(*line))
+	while (*line && isspace((int)*line))
 	    line++;
 
 	if (!*line)
@@ -549,7 +553,7 @@ static int interpret_line(char *line, ADDRESS *addr, int mode)
  * error code is returned.
  */
 
-static int interpret_instr(char *line, ADDRESS adr, int mode)
+int interpret_instr(char *line, ADDRESS adr, int mode)
 {
     int     i = 0;
     int     instr = 0;
@@ -603,21 +607,21 @@ int find_instr(char *line)
 {
     int i;
 
-    while (*line && isspace(*line))
+    while (*line && isspace((int)*line))
         line++;
- 
+
     if (!*line)
         return (-1); /* line doesn't contain any mnemonic */
- 
+
     for (i = 0; line[i] && i < 3; i++) {
-        if (isalpha (line[i]))
+        if (isalpha ((int)line[i]))
             line[i] = toupper (line[i]);
-    }       
- 
-        for (i = 0; i < TOTAL_CODES; i++)
-            if (0 == strncmp(lookup[i].mnemonic, line, 3) &&
-		(lookup[i].mnemonic[3] < ' ' ||	lookup[i].mnemonic[3] == line[3]))
-		return (i);		/* USBC, NOOP, RBMn ... */
+    }
+
+    for (i = 0; i < TOTAL_CODES; i++)
+        if (0 == strncmp(lookup[i].mnemonic, line, 3) &&
+            (lookup[i].mnemonic[3] < ' ' ||	lookup[i].mnemonic[3] == line[3]))
+            return (i);		/* USBC, NOOP, RBMn ... */
 
     return (-1);
 }
@@ -637,7 +641,7 @@ static int parse_arguments(ADDRESS adr, int code, char *line, BYTE *arg, int *le
     int     i, ival;
     char   *linep;
 
-    while (line && *line && isspace(*line))
+    while (line && *line && isspace((int)*line))
 	line++;
 
     if (NULL == line || !*line) {	/* IMPLIED addressing mode. */
@@ -761,7 +765,7 @@ static int parse_arguments(ADDRESS adr, int code, char *line, BYTE *arg, int *le
 #endif
 		}  /* switch */
 
-		if (*linep && !isspace(*linep) && *linep != ')')
+		if (*linep && !isspace((int)*linep) && *linep != ')')
 		    return E_BAD_INDEX;
 	    }
 	}
@@ -903,3 +907,4 @@ static int parse_arguments(ADDRESS adr, int code, char *line, BYTE *arg, int *le
 
     return E_SYNTAX;	/* Illegal address mode */
 }
+#endif
