@@ -5,11 +5,17 @@
  *  Teemu Rantanen   (tvr@cs.hut.fi)
  *  Jouko Valta      (jopi@zombie.oulu.fi)
  *  Gerhard Wesp     (gwesp@cosy.sbg.ac.at)
+ *  Daniel Sladic    (sladic@eecg.toronto.edu)
+ *  Ricardo Ferreira (storm@esoterica.pt)
+ *  Andreas Boose    (boose@unixserv.rz.fh-hannover.de)
  *
  * Patches by
  *  Olaf Seibert     (rhialto@mbfys.kun.nl)
  *  Ettore Perazzoli (ettore@comm2000.it)
  *
+ * Zipcode implementation based on `zip2disk' by
+ *  Paul David Doherty (h0142kdd@rz.hu-berlin.de)
+ *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
  *
@@ -25,7 +31,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
@@ -53,15 +60,20 @@
 #include "mshell.h"
 #include "serial.h"
 #include "drive.h"
+#include "gcr.h"
 #include "file.h"
 #include "charsets.h"
 #include "tape.h"
+#include "utils.h"
+#include "zipcode.h"
 
 #define MAXARG		16
 #define MAXVAL		0xffff
 
 #define MAXDRIVE	1
 
+#define C1541_VERSION_MAJOR	2
+#define C1541_VERSION_MINOR	4
 
 /* Global */
 
@@ -87,6 +99,7 @@ static void usage( void );
 static int  disk_attach ( void );
 static int  disk_cdd ( void );
 static int  disk_format ( void );
+static int  disk_gcrformat ( void );
 static int  disk_delete ( void );
 static int  disk_list ( void );
 static int  disk_validate ( void );
@@ -95,6 +108,8 @@ static int  disk_rename ( void );
 static int  disk_read ( void );
 static int  disk_write ( void );
 static int  disk_import ( void );
+static int  disk_import_zipfile ( void );
+static int  disk_unlynx ( void );
 static int  disk_copy_tape ( void );
 static int  disk_sectordump ( void );
 static int  disk_extract ( void );
@@ -102,11 +117,16 @@ static int  disk_raw_command ( void );
 static int  disk_info ( void );
 static int  disk_help ( void );
 static int  disk_quit ( void );
+static int  disk_system ( void );
 
+extern char sector_map[43]; /* Ugly: FIXME! */
+extern int speed_map[42];
 
 struct ms_table disk_cmds[] = {
     {"format", 1, 2, disk_format,
-    "format  [imagename] 'diskname,id'\t(create disk & format)\n"},
+    "format  [imagename] 'diskname,id'\t(create X64 disk & format)\n"},
+    {"gcrformat", 1, 2, disk_gcrformat,
+    "gcrformat  imagename ['diskname,id']\t(create GCR disk & format)\n"},
     {"delete", 1, MAXARG, disk_delete,
     "delete  files\n"},
     {"list", 0, MAXARG, disk_list,
@@ -131,9 +151,13 @@ struct ms_table disk_cmds[] = {
      "unit\t  drive\t\t\t\t(changes default device)\n"},
 
     {"create", 2, 3, disk_import,
-    "create  imagename  dumpname ['descr']\t(create disk & import a raw diskdump)\n"},
+    "create  imagename dumpname ['descr']\t(create disk & import a raw diskdump)\n"},
+    {"zcreate", 2, 3, disk_import_zipfile,
+    "zcreate  imagename zipname ['descr']\t(create disk & import zipfile images)\n"},
     {"tape", 1, MAXARG, disk_copy_tape,
     "tape\t  tapeimage [files]\t\t(copy files from tape image)\n"},
+    {"unlynx", 1, 1, disk_unlynx,
+     "unlynx lynxname\t\t\t(unarchive an LNX file to disk image)\n"},
     {"read", 1, 2, disk_read,
     "read\t  1541name [FSname]\t\t(copy file from disk image)\n"},
     {"write", 1, 2, disk_write,
@@ -145,6 +169,7 @@ struct ms_table disk_cmds[] = {
     {"hunt", 3, MAXARG, disk_hunt,
     "hunt     start end|+len  bytes|'string'\n"},
 #endif
+
     {"block", 0, 4, disk_sectordump,
     "block [drive:] [track] [sector] [disp] (show disk blocks in hex format)\n"},
 
@@ -155,6 +180,8 @@ struct ms_table disk_cmds[] = {
 
     {"@", 0, 1, disk_raw_command,
     "@\t[disk command]\t\t\t(command for channel 15)\n"},
+    {"system", 1, 1, disk_system,
+     "system command\t\t\t(execute a system command)\n"},
     {"help", 0, 1, disk_help,
     "help\t [topic]\t\t\t(Describe command)\n"},
     {"?", 0, 1, disk_help,
@@ -384,6 +411,138 @@ static int  disk_cdd(void)
     return (0);
 }
 
+/*
+ * Creates a raw GCR disk image.
+ */
+
+static int  disk_gcrformat (void)
+{
+    int fd, track, sector;
+    BYTE gcr_header[12], id[2];
+    char name[16], *idptr;
+    DWORD gcr_track_p[MAX_TRACKS_1541 * 2];
+    DWORD gcr_speed_p[MAX_TRACKS_1541 * 2];
+    BYTE gcr_track[7930], rawdata[260];
+    BYTE *gcrptr;
+
+    if (nargs < 2)
+	return 0;
+
+    memset(name, 0xa0, 16);
+    id[0] = id[1] = 0xa0;
+
+    if (nargs > 2) {
+	strcpy(newname, args[2]);
+	petconvstring(newname, 1);
+	idptr = memchr(newname, ',', strlen(newname));
+	if (idptr == NULL) {
+	    int i;
+	    for (i = 0; i < 16 && newname[i] != '\0'; i++)
+		name[i] = newname[i];
+	} else {
+	    int i;
+	    *idptr = '\0';
+	    for (i = 0; i < 16 && newname[i] != '\0'; i++)
+		name[i] = newname[i];
+	    strncpy(id, ++idptr, 2);
+	    if (idptr[1] != '\0') {
+		id[0] = idptr[1];
+		if (idptr[2] != '\0')
+		    id[1] = idptr[2];
+	    }
+	}
+    }
+
+    if ((fd = open(args[1], O_RDWR | O_CREAT, 0666)) < 0) {
+	printf("could not create image %s\n", args[1]);
+	return (FD_BADIMAGE);
+    }
+
+    strcpy(gcr_header, "GCR-1541");
+
+    gcr_header[8] = 0;
+    gcr_header[9] = MAX_TRACKS_1541 * 2;
+    gcr_header[10] = 7928 % 256;
+    gcr_header[11] = 7928 / 256;
+
+    if (write(fd, (char *)gcr_header, sizeof(gcr_header))
+        != sizeof(gcr_header)) {
+	printf("Cannot write header.\n");
+	close(fd);
+	return 0;
+    }
+
+    for (track = 0; track < MAX_TRACKS_1541; track++) {
+	gcr_track_p[track * 2] = 12 + MAX_TRACKS_1541 * 16 + track * 7930;
+	gcr_track_p[track * 2 + 1] = 0;
+	gcr_speed_p[track * 2] = speed_map[track];
+	gcr_speed_p[track * 2 + 1] = 0;
+    }
+
+    if (write_dword(fd, gcr_track_p, sizeof(gcr_track_p)) < 0) {
+	printf("Cannot write track header.\n");
+	close(fd);
+	return 0;
+    }
+
+    if (write_dword(fd, gcr_speed_p, sizeof(gcr_speed_p)) < 0) {
+	printf("Cannot write speed header.\n");
+	close(fd);
+	return 0;
+    }
+
+    for (track = 0; track < MAX_TRACKS_1541; track++) {
+
+	int raw_track_size[4] = { 6250, 6666, 7142, 7692 };
+
+	memset(&gcr_track[2], 0xff, 7928);
+	gcr_track[0] = raw_track_size[speed_map[track]] % 256;
+	gcr_track[1] = raw_track_size[speed_map[track]] / 256;
+	gcrptr = &gcr_track[2];
+
+	for (sector = 0; sector < sector_map[track+1]; sector++) {
+	    BYTE chksum;
+	    int i;
+	    memset(rawdata, 0, 260);
+	    if (track == 17 && sector == 0) {
+		BYTE *rdat = &rawdata[1];
+		int s, t;
+		memset(rdat + BAM_DISK_NAME, 0xa0, 27);
+		rdat[0] = DSK_DIR_TRACK;
+		rdat[1] = DSK_DIR_SECTOR;
+		rdat[2] = 65;
+		rdat[BAM_VERSION] = 50;
+		rdat[BAM_VERSION + 1] = 65;
+		memcpy(rdat + BAM_DISK_NAME, (BYTE *)name, 16);
+		rdat[BAM_DISK_ID] = id[0];
+		rdat[BAM_DISK_ID + 1] = id[1];
+		for (t = 1; t <= 35; t++)
+		    for (s = 0; s < sector_map[t]; s++)
+			free_sector(rdat, t, s);
+		allocate_sector(rdat, DSK_BAM_TRACK, 0);
+		allocate_sector(rdat, DSK_BAM_TRACK, 1);
+	    }
+	    rawdata[0] = 7;
+	    chksum = rawdata[1];
+	    for (i = 1; i < 256; i++)
+		chksum ^= rawdata[i + 1];
+	    rawdata[257] = chksum;
+
+	    convert_sector_to_GCR(rawdata, gcrptr, track+1, sector, id[0], id[1]);
+	    gcrptr += 360;
+	}
+
+	if (write(fd, (char *)gcr_track, sizeof(gcr_track))
+            != sizeof(gcr_track)) {
+	    printf("Cannot write track data.\n");
+	    close(fd);
+	    return 0;
+	}
+    }
+
+    close(fd);
+    return 0;
+}
 
 /*
  * Simulate the 1541 DOS commands
@@ -467,7 +626,7 @@ static int  disk_list (void)
     DRIVE *floppy;
     int   dev, err, i = 1;
     char *name, *listing;
-    
+
     do {
 	name = args[i];
 	dev =  find_devno(DriveNum, name);
@@ -644,7 +803,7 @@ static int  disk_read (void)
      */
 
     memcpy (newname, floppy->buffers[0].slot + SLOT_NAME_OFFSET, 16);
-    if (nargs > 2) 
+    if (nargs > 2)
 	fsname = args[2];
     else {
 	int l;
@@ -774,6 +933,124 @@ static DiskFormats  Legal_formats[] = {
   {  -1,     0, 0,    0 }
 };
 
+static int  disk_import_zipfile (void)
+{
+    DRIVE  *floppy = DriveData[DriveNum];
+    DiskFormats  *format = Legal_formats;
+    char    tmp[256];
+    int     fsfd=-1, errblk = 0;
+    int track, sector, count;
+    char *fname;
+    int    channel = 2;
+    BYTE  str[20];
+    static int  drive = 8;
+    int		singlefilemode = 0;
+
+    /*
+     * Open image or create a new one.
+     * If the file exists, it must have valid header.
+     */
+    if (open_image(DriveNum, args[1], 1) < 0) {
+	return (FD_BADIMAGE);
+    }
+
+    fname = (char *) malloc (sizeof(char) * strlen(args[2]) + 3);
+    /* ignore '[0-4]!' if found */
+    if (args[2][0] >= '1' && args[2][0] <= '4' && args[2][1] == '!')
+	strcpy(fname+2, args[2]+2);
+    else
+	strcpy(fname+2, args[2]);
+    fname[0] = '0';
+    fname[1] = '!';
+
+    set_label(floppy->ActiveFd, "*** Truncated image."); /* Notify of errors */
+
+    printf("copying blocks to image\n");
+    lseek(floppy->ActiveFd, HEADER_LENGTH, SEEK_SET);
+
+    /* Write out all the sectors */
+    for(count=0;count<format->TotalBks;count++) {
+       if (write(floppy->ActiveFd, tmp, 256) != 256) {
+          printf("cannot write block %d of %s\n", count, args[2]);
+          return (FD_WRTERR);
+       }
+    }
+
+    if (open_1541(floppy, "#", 1, channel)) {
+       printf("u%d: cannot open buffer #%d.\n", drive, channel);
+       return (FD_RDERR);
+    }
+
+    for (track = 1; track <= 35; track++) {
+	if (singlefilemode || track == 1) {
+	    if (track == 1) {
+		fsfd = open(fname + 2, O_RDONLY);
+		if (fsfd >= 0) {
+		    printf("reading zipfile on onefile -mode\n");
+		    singlefilemode = 1;
+		    lseek(fsfd, 4, SEEK_SET);
+		}
+	    } else if (track == 9 || track == 17 || track == 26) {
+		lseek(fsfd, 2, SEEK_CUR);
+	    }
+	}
+	if (!singlefilemode) {
+	    switch (track) {
+	    case 1:
+	    case 9:
+	    case 17:
+	    case 26:
+		fname[0]++;
+		if (fsfd >= 0)
+		    close(fsfd);
+		if ((fsfd = open(fname, O_RDONLY)) < 0) {
+		    printf("cannot open %s\n", fname);
+		    perror (fname);
+		    return (FD_NOTRD);
+		}
+		lseek(fsfd, (track == 1) ? 4 : 2, SEEK_SET);
+		break;
+	    }
+	}
+
+       for (count = 0; count < sector_map[track]; count++) {
+          if ( (zipcode_read_sector(fsfd, track, &sector, floppy->buffers[channel].buffer)) != 0) {
+	      close(fsfd);
+	      return (FD_BADIMAGE);
+          }
+
+          /* Write one block */
+
+          sprintf((char *)str, "B-W:%d 0 %d %d", channel, track, sector);
+          if (ip_execute(floppy, (BYTE *)str, strlen((char *)str)) != 0) {
+	      track = DSK_DIR_TRACK;
+	      sector= 0;
+	      close(fsfd);
+	      return (FD_RDERR);
+          }
+       }
+    }
+    close_1541(floppy, channel);
+
+    /* Update Format and Label information on Disk Header */
+
+    lseek(floppy->ActiveFd, (off_t)HEADER_LABEL_OFFSET + 0, SEEK_SET);
+
+    if (write(floppy->ActiveFd, &(format->ImageFormat), 1) != 1) {
+	return (FD_WRTERR);
+    }
+
+    set_disk_size(floppy->ActiveFd, format->TracksSide, format->Sides, errblk);
+
+    set_label(floppy->ActiveFd, (args[3] ? args[3] : NULL)); /* Fix the note */
+
+    close(fsfd);
+
+    ip_execute(floppy, (BYTE *)"I", 1);
+
+    return(0);
+}
+
 
 static int  disk_import (void)
 {
@@ -817,7 +1094,7 @@ static int  disk_import (void)
     }
 
 
-    /* Now recognice the format and verify block count on it. */
+    /* Now recognize the format and verify block count on it. */
 
     if (blk <  NUM_BLOCKS_1541) {
 	    printf("cannot read block %d of %s\n", blk, args[2]);
@@ -1103,13 +1380,13 @@ static int  disk_extract(void)
     if ((err = check_drive(drive, CHK_RDY)) < 0)
 	return (err);
     floppy = DriveData[drive & 3];
-      
+
     if (open_1541(floppy, "#", 1, channel)) {
 	printf("u%d: cannot open buffer #%d.\n", drive, channel);
 	return (FD_RDERR);
     }
 
-    while(1) 
+    while(1)
     {
 	int i;
 	/* Read one block */
@@ -1132,8 +1409,8 @@ static int  disk_extract(void)
 		FILE *fd;
 		BYTE name[17];
 		BYTE *l= memccpy(name,entry->FileName,0xa0,16);
-          
-		if(l) {
+
+		if (l) {
 		    *(l - 1)= 0;	/* terminate by 0 */
 		    len= l - 1 - name;
 		} else {
@@ -1142,7 +1419,7 @@ static int  disk_extract(void)
 		}
 
 		petconvstring(name,1);
-		printf("%s\n",name);  
+		printf("%s\n",name);
 
 		unix_filename(name);	/* for now, convert '/' to '_' */
 
@@ -1151,18 +1428,18 @@ static int  disk_extract(void)
 		    continue;
 		}
 
-		if(!(fd= fopen(name,"wb"))) {
+		if (!(fd= fopen(name,"wb"))) {
 		    perror("Couldn't open unix file");
 		    close_1541(floppy,0);
 		    continue;
-		} 
-          
+		}
+
 		while (!read_1541(floppy,&c,0))
 		    fputc(c,fd);
-         
+
 		close_1541(floppy,0);
-          
-		if(fclose(fd)) {
+
+		if (fclose(fd)) {
 		    perror("fclose");
 		    return FD_RDERR;
 		}
@@ -1252,10 +1529,6 @@ static int  disk_info (void)
 	return (set_label(floppy -> ActiveFd, args[1]));
     }
     else {
-	/*printf("Disk image version %d.%02d, emulator format V %d.%02d\n",
-	       hdr.v_major, hdr.v_minor,
-	       HEADER_VERSION_MAJOR, HEADER_VERSION_MINOR);*/
-
 	printf ("\nDescription: %s\n",
 		(*hdr.description ? hdr.description : "None."));
 	printf ("Drive Type : %d.\n", floppy->ImageFormat);  /* Compatible drive */
@@ -1279,9 +1552,11 @@ static void usage()
     -validate image                Validate disk\n    %s\n\
     -create image dump ['descr']   Build image out of raw blockdump\n\
                                     (683 256-byte blocks, e.g. *.d64 file)\n\
+    -zcreate image zipfile         Create image out of a ZIPcoded file\n\
+    -unlynx image lynxfile         Extract contents of a LYNX file\n\
     -tape image tapeimage [files]  Transfer files from *.t64 image\n\
     -read image 1541name [FSname]  Read 1541name on image to file FSname\n\
-    -extract image                  Extract all files on image\n\
+    -extract image                 Extract all files on image\n\
     -write image FSname [1541name] Write bin or P00 file FSname to image\n\
     -block [drive:] [track] [sector] [disp] \n\
                                    Show disk blocks in hex format\n\
@@ -1328,7 +1603,179 @@ static int  disk_help(void)
 
 static int  disk_quit(void)
 {
-    return (1);
+    return 1;
+}
+
+
+/* Lynx support added by Riccardo Ferreira (storm@esoterica.pt) --
+   1998-02-07.  */
+static int disk_unlynx(void)
+{
+    DRIVE *floppy;
+    FILE *f, *f2;
+    int err, dev, cnt=0;
+    long dentries, lbsize, bsize, dirsize;
+    BYTE val;
+    char buff[256] = { 0 }, cname[20] = { 0 }, ftype;
+
+    if (!(f=fopen(args[1], READ))) {
+	printf("cannot open %s for reading\n", args[1]);
+	return (FD_NOTRD);
+    }
+
+    /* Look for the 0, 0, 0 sign of the end of BASIC */
+    while (1) {
+	fread(&val, 1, 1, f);
+	if (val == 0)
+	    cnt++;
+	else
+	    cnt = 0;
+	if (cnt == 3)
+	    break;
+    }
+
+    /* Bypass the 1st return in the file */
+    fgetc(f);
+
+    /* Get the directory block size */
+    cnt = 0;
+    while (1) {
+	fread(&val, 1, 1, f);
+	if (val != 13)
+	    buff[cnt++] = val;
+	else
+	    break;
+    }
+    buff[cnt] = 0;
+    if (string_to_long(buff, NULL, 10, &dirsize) < 0
+	|| dirsize <= 0) {
+	printf("invalid LyNX file.\n");
+	return (FD_RDERR);
+    }
+
+    /* Get the number of dir entries */
+    cnt = 0;
+    while (1) {
+	fread(&val, 1, 1, f);
+	if (val != 13)
+	    buff[cnt++] = val;
+	else
+	    break;
+    }
+    buff[cnt] = 0;
+    if (string_to_long(buff, NULL, 10, &dentries) < 0
+	|| dentries <= 0) {
+	printf("invalid LyNX file.\n");
+	return (FD_RDERR);
+    }
+
+    /* Open the file for reading of the chained data */
+    f2 = fopen(args[1], READ);
+    fseek(f2, (dirsize*254), SEEK_SET);
+
+    /* Loop */
+    while (dentries != 0) {
+	/* Read CBM filename */
+	cnt = 0;
+	while (1) {
+	    fread(&val, 1, 1, f);
+	    if (val != 13) cname[cnt++] = val; else break;
+	}
+	cname[cnt] = 0;
+
+	/* Read the block size */
+	cnt = 0;
+	while (1) {
+	    fread(&val, 1, 1, f);
+	    if (val != 13) buff[cnt++] = val; else break;
+	}
+	buff[cnt] = 0;
+
+	if (string_to_long(buff, NULL, 10, &bsize) < 0) {
+	    printf("invalid LyNX file.\n");
+	    return (FD_RDERR);
+	}
+
+	/* Get the file type (P[RG], S[EQ], R[EL], U[SR]) */
+	ftype = fgetc(f);
+	fgetc(f);
+
+	/* FIXME: REL type files unsupported */
+	if (ftype == 'R') {
+	    printf("REL not supported.\n");
+	    return (FD_RDERR);
+	}
+
+	/* FIXME: This is a temporary hack!  How can we persuade `open_1541()'
+	   to write `DEL' files without breaking compatibility with CBM
+	   DOS?  -- [EP] 98.04.17  */
+	if (ftype == 'D')
+	    ftype = 'P';
+
+	/* Add the file type to the name */
+	cnt = strlen(cname);
+	cname[cnt++] = ',';
+	cname[cnt++] = ftype;
+	cname[cnt] = 0;
+
+	/* Get the byte size of the last block +1 */
+	cnt = 0;
+	while (1) {
+	    fread(&val, 1, 1, f);
+	    if (val != 13)
+		buff[cnt++] = val;
+	    else
+		break;
+	}
+	buff[cnt] = 0;
+
+	if (string_to_long(buff, NULL, 10, &lbsize) < 0) {
+	    printf("invalid LyNX file.\n");
+	    return (FD_RDERR);
+	}
+
+	/* Calculate byte size of file */
+	cnt = bsize*254;
+
+	/* Get the device number */
+	dev = find_devno(DriveNum, (char *)NULL);
+
+	if ((err = check_drive(dev, CHK_RDY))<0)
+	    return(err);
+
+	floppy = DriveData[dev & 3];
+
+	if (open_1541(floppy, cname, strlen(cname), 1)) {
+	    printf("couldn't open '%s' for writing.\n", cname);
+	    return (FD_WRTERR);
+	}
+
+	printf("writing file '%s' to image.\n", cname);
+
+	while (cnt != 0) {
+	    fread(&val, 1, 1, f2);
+	    if (write_1541(floppy, val, 1)) {
+		printf("no space on image ?\n");
+		break;
+	    }
+	    cnt--;
+	}
+	close_1541(floppy, 1);
+
+	/* Adjust for the last block */
+	fread(buff, 1, 254 - lbsize, f2);
+	dentries--;
+    }
+    fclose(f);
+    fclose(f2);
+    return(0);
+}
+
+/* `system' command added by Riccardo Ferreira (storm@esoterica.pt) --
+   1998-02-07.  */
+int disk_system(void)
+{
+	return system(args[1]);
 }
 
 
@@ -1347,8 +1794,8 @@ int     main(argc, argv)
     progname = argv[0];
     nargs    = 0;
 
-    initialize_1541(8, DT_DISK | DT_1541);
-    initialize_1541(9, DT_DISK | DT_1541);
+    initialize_1541(8, DT_DISK | DT_1541, NULL, NULL, NULL);
+    initialize_1541(9, DT_DISK | DT_1541, NULL, NULL, NULL);
 
 
     /* Direct mode */
@@ -1362,7 +1809,8 @@ int     main(argc, argv)
 	args[0]  = argv[1] + 1;				/* Command */
 	ImageName = argv[2];
 
-	if (!strncmp(args[0], "f", 1) || !strncmp(args[0], "cr", 2)) {
+	if (!strncmp(args[0], "f", 1) || !strncmp(args[0], "cr", 2) ||
+	    !strncmp(args[0], "zcr", 3)) {
 	    --argc;
 	    ++argv;
 	} else {
@@ -1483,3 +1931,21 @@ int     main(argc, argv)
     printf("\n");
     return (0);
 }
+
+int attach_fsdevice(int device, char *var, char *name)
+{
+    DriveData[device & 3] = (DRIVE *)var;
+    return 0;
+}
+
+int serial_attach_device(int device, char *var, char *name,
+                         int (*getf)(void *, BYTE * , int),
+                         int (*putf)(void *, BYTE , int),
+                         int (*openf)(void *, char *, int , int),
+                         int (*closef)(void *, int),
+                         void (*flushf)(void *, int))
+{
+    DriveData[device & 3] = (DRIVE *)var;
+    return 0;
+}
+
