@@ -2,11 +2,11 @@
  * cia-tmpl.c - Template file for MOS6526 (CIA) emulation.
  *
  * Written by
- *   Andre' Fachat (fachat@physik.tu-chemnitz.de)
+ *  Andr Fachat (fachat@physik.tu-chemnitz.de)
  *
  * Patches and improvements by
- *   Ettore Perazzoli (ettore@comm2000.it)
- *   Andreas Boose (boose@rzgw.rz.fh-hannover.de)
+ *  Ettore Perazzoli (ettore@comm2000.it)
+ *  Andreas Boose (boose@rzgw.rz.fh-hannover.de)
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -23,44 +23,55 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
 /*
  * 03jun1997 a.fachat
- * complete timer rewrite 
+ * complete timer rewrite
  *
  * There now is a new function, update_mycia(). It computes all differences
- * from one given state to a new state at rclk. 
- * Well, not everything. For the cascade mode (timer B counting timer A)
- * as well as the shift register the interrupt function is being used.
+ * from one given state to a new state at rclk.  Well, not everything. For
+ * the cascade mode (timer B counting timer A) as well as the shift register
+ * the interrupt function is being used.
  *
- * The update routine computes, completely independent from the 
- * interrupt stuff, the timer register values. The update routines are
- * only called when a timer register is touched.
- * int_*() is now called only _once_ per interrupt, and it is much smaller
- * than the last version. It is disabled when the interrupt flag
- * is not set (at least timer B. For timer A there must not be 
- * a shift register operation and not timer B counting timer A) 
+ * The update routine computes, completely independent from the interrupt
+ * stuff, the timer register values. The update routines are only called when
+ * a timer register is touched.  int_*() is now called only _once_ per
+ * interrupt, and it is much smaller than the last version. It is disabled
+ * when the interrupt flag is not set (at least timer B. For timer A there
+ * must not be a shift register operation and not timer B counting timer A)
  *
- * The timer B when counting timer A and the Shift register interrupt flags 
- * may be set a few cycles too late due to late call of int_myciat*() due 
- * to opcode execution time. This can be fixed by checking in the beginning
- * of read_* and write_* if an int_* is scheduled and executing it before.
- * Then the setting of the ICR could also be moved from update to int_*().
- * But the bug only affects the contents of the ICR. The interrupt is 
- * generated at the right time (hopefully).
+ * The timer B when counting timer A and the Shift register interrupt flags
+ * may be set a few cycles too late due to late call of int_myciat*() due to
+ * opcode execution time. This can be fixed by checking in the beginning of
+ * read_* and write_* if an int_* is scheduled and executing it before.  Then
+ * the setting of the ICR could also be moved from update to int_*().  But
+ * the bug only affects the contents of the ICR. The interrupt is generated
+ * at the right time (hopefully).
  *
- * There are one HACK to make a game work: in update_mycia() a fix is done
- * for arkanoid. This game counts shift register bits (i.e. TA underflows) by
+ * There is one HACK to make a game work: in update_mycia() a fix is done for
+ * Arkanoid. This game counts shift register bits (i.e. TA underflows) by
  * setting TA to one-shot.  The ICR is read just before the int_myciata()
  * function is called, and the int bit is missed, so there is a check in
  * update_mycia() (this is probably a fix and not a hack... :-)
  *
- * Still some work has to be put into the dump functions, to really show
- * the state of the chip and not the state of the emulation. I.e. we
- * need to compute the bits described in the emulator test suite.  */
+ * Still some work has to be put into the dump functions, to really show the
+ * state of the chip and not the state of the emulation. I.e. we need to
+ * compute the bits described in the emulator test suite.
+ *
+ */
+
+/*
+ * 29jun1998 a.fachat
+ *
+ * Implementing the peek function assumes that the READ_PA etc macros
+ * do not have side-effects, i.e. they can be called more than once
+ * at one clock cycle.
+ *
+ */
 
 #include "vice.h"
 
@@ -71,7 +82,7 @@
 #include "vmachine.h"
 #include "cia.h"
 #include "vicii.h"
-#include "maincpu.h"
+#include "mycpu.h"
 #include "resources.h"
 
 INCLUDES
@@ -88,34 +99,86 @@ INCLUDES
 #define	CIAT_RUNNING	1
 #define	CIAT_COUNTTA	2
 
-/* FIXME: the `force_interrupt_check()' call is sometimes redundant. */
 #ifdef MYCIA_TIMER_DEBUG
 #define	my_set_int(int_num, value, rclk)				\
     do {								\
-        if (app_resources.debugFlag)					\
-	    printf("set_int(rclk=%d, int=%d, d=%d pc=%04x)\n",		\
-		   rclk,(int_num),(value),*reg_pcp);			\
+        if (mycia_debugFlag)					\
+	    printf("set_int(rclk=%d, int=%d, d=%d pc=)\n",		\
+		   rclk,(int_num),(value));			\
 	mycia_set_int_clk((int_num), (value), (rclk));			\
-	if ((value)) {							\
-	    maincpu_force_interrupt_check();				\
+	if ((value))							\
 	    myciaint |= 0x80;						\
-	}								\
     } while(0)
 #else
 #define	my_set_int(int_num, value, rclk)				 \
     do {								 \
         mycia_set_int_clk((int_num), (value), (rclk));			 \
-	if ((value)) {							 \
-	    maincpu_force_interrupt_check();				 \
+	if ((value))							 \
 	    myciaint |= 0x80;						 \
-	}								 \
     } while(0)
 #endif
 
+/* 
+ * scheduling int_myciat[ab] calls - 
+ * warning: int_myciata uses mycpu_* stuff! 
+ */
+
+#define	my_set_tai_clk(clk) 						\
+    do {								\
+	mycia_tai = clk;						\
+	mycpu_set_alarm_clk(A_MYCIATA, clk);				\
+    } while(0)
+
+#define	my_unset_tai() 							\
+    do {								\
+	mycia_tai = -1;							\
+	mycpu_unset_alarm(A_MYCIATA);					\
+    } while(0)
+
+#define	my_set_tbi_clk(clk) 						\
+    do {								\
+	mycia_tbi = clk;						\
+	mycpu_set_alarm_clk(A_MYCIATB, clk);				\
+    } while(0)
+
+#define	my_unset_tbi() 							\
+    do {								\
+	mycia_tbi = -1;							\
+	mycpu_unset_alarm(A_MYCIATB);					\
+    } while(0)
+
+/*
+ * Those routines setup the myciat[ab]i clocks to a value above
+ * rclk and schedule the next int_myciat[ab] alarm
+ */
+#define	update_tai(rclk)							\
+    do {								\
+	if(mycia_tai < rclk) {						\
+	    int t = myciaint;						\
+	    myciaint = 0;						\
+	    int_myciata(rclk - mycia_tai);				\
+	    myciaint |= t;						\
+	}								\
+    } while(0)
+
+#define	update_tbi(rclk)							\
+    do {								\
+	if(mycia_tbi < rclk) {						\
+	    int t = myciaint;						\
+	    myciaint = 0;						\
+	    int_myciatb(rclk - mycia_tbi);				\
+	    myciaint |= t;						\
+	}								\
+    } while(0)
+
 /* global */
 
-BYTE mycia[16];
-extern unsigned int *reg_pcp;
+static BYTE mycia[16];
+
+#if defined(MYCIA_TIMER_DEBUG) || defined(MYCIA_IO_DEBUG)
+int mycia_debugFlag = 0;
+
+#endif
 
 /* local functions */
 
@@ -134,6 +197,7 @@ static int myciaint;		/* Interrupt Flag register for cia 1 */
 static CLOCK myciardi;		/* real clock = clk-offset */
 
 static CLOCK mycia_tau;		/* when is the next underflow? */
+static CLOCK mycia_tai;		/* when is the next int_* scheduled? */
 static unsigned int mycia_tal;	/* latch value */
 static unsigned int mycia_tac;	/* counter value */
 static unsigned int mycia_tat;	/* timer A toggle bit */
@@ -141,6 +205,7 @@ static unsigned int mycia_tap;	/* timer A port bit */
 static int mycia_tas;		/* timer state (CIAT_*) */
 
 static CLOCK mycia_tbu;		/* when is the next underflow? */
+static CLOCK mycia_tbi;		/* when is the next int_* scheduled? */
 static unsigned int mycia_tbl;	/* latch value */
 static unsigned int mycia_tbc;	/* counter value */
 static unsigned int mycia_tbt;	/* timer B toggle bit */
@@ -155,6 +220,9 @@ static char myciatodstopped;
 static char myciatodlatched;
 static int myciatodticks = 100000;	/* approx. a 1/10 sec. */
 
+static BYTE myciaflag = 0;
+
+
 /* ------------------------------------------------------------------------- */
 /* MYCIA */
 
@@ -179,7 +247,7 @@ static int update_mycia(CLOCK rclk)
     CLOCK added_int_clk = (myciaint & 0x80) ? rclk - 3 : CLOCK_MAX;
 
 #ifdef MYCIA_TIMER_DEBUG
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("MYCIA: update: rclk=%d, tas=%d, tau=%d, tal=%u, ",
 	       rclk, mycia_tas, mycia_tau, mycia_tal);
 #endif
@@ -195,7 +263,7 @@ static int update_mycia(CLOCK rclk)
 		    && (mycia_tau < added_int_clk))
 		    added_int_clk = mycia_tau;
 		mycia_tau = 0;
-		maincpu_unset_alarm(A_MYCIATA);
+		my_unset_tai();
 		mycia_tac = mycia_tal;
 		mycia_tas = CIAT_STOPPED;
 		mycia[CIA_CRA] &= 0xfe;
@@ -203,6 +271,9 @@ static int update_mycia(CLOCK rclk)
 		/* this is a HACK for arkanoid... */
 		if (myciasr_bits) {
 		    myciasr_bits--;
+		    if(myciasr_bits==16) {
+			STORE_SDR
+		    }
 		    if (!myciasr_bits) {
 			myciaint |= CIA_IM_SDR;
 			if ((myciaier & CIA_IM_SDR)
@@ -220,13 +291,14 @@ static int update_mycia(CLOCK rclk)
 		mycia_tac = mycia_tau - rclk;
 	    }
 
-	    myciaint |= CIA_IM_TA;
 	    if (mycia_tac == mycia_tal)
 		ista = 1;
+
+            myciaint |= CIA_IM_TA;
 	}
     }
 #ifdef MYCIA_TIMER_DEBUG
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("aic=%d, tac-> %u, tau-> %d\n              tmp=%u, ", added_int_clk, mycia_tac, mycia_tau, tmp);
 #endif
 
@@ -246,7 +318,7 @@ static int update_mycia(CLOCK rclk)
 		if ((myciaier & CIA_IM_TB) && (mycia_tbu < added_int_clk))
 		    added_int_clk = mycia_tbu;
 		mycia_tbu = 0;
-		maincpu_unset_alarm(A_MYCIATB);
+		my_unset_tbi();
 		mycia_tbc = mycia_tbl;
 		mycia_tbs = CIAT_STOPPED;
 		mycia[CIA_CRB] &= 0xfe;
@@ -260,8 +332,7 @@ static int update_mycia(CLOCK rclk)
 	    }
 	    if (!mycia_tbc)
 		mycia_tbc = mycia_tbl;
-
-	    myciaint |= CIA_IM_TB;
+            myciaint |= CIA_IM_TB;
 	}
     } else if (mycia_tbs == CIAT_COUNTTA) {
 	/* missing: set added_int */
@@ -278,7 +349,7 @@ static int update_mycia(CLOCK rclk)
     }
 
 #ifdef MYCIA_TIMER_DEBUG
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("tbc-> %u, tbu-> %d, int %02x ->",
 	       mycia_tbc, mycia_tbu, myciaint);
 #endif
@@ -295,7 +366,7 @@ static int update_mycia(CLOCK rclk)
 	    if (added_int_clk == myciardi) {
 		myciaint &= 0x7f;
 #ifdef MYCIA_TIMER_DEBUG
-		if (app_resources.debugFlag)
+		if (mycia_debugFlag)
 		    printf("MYCIA: TA Reading ICR at rclk=%d prevented IRQ\n",
 			   rclk);
 #endif
@@ -308,7 +379,7 @@ static int update_mycia(CLOCK rclk)
 	}
     }
 #ifdef MYCIA_TIMER_DEBUG
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("%02x\n", myciaint);
 #endif
 
@@ -321,55 +392,49 @@ static int update_mycia(CLOCK rclk)
 void reset_mycia(void)
 {
     int i;
+
     for (i = 0; i < 16; i++)
 	mycia[i] = 0;
 
     myciardi = 0;
     myciasr_bits = 0;
 
-    /* FIXME: Which is the correct version? */
-#if 0
-    /* This is according to Wolfgang Lorentz' CIA suite, breaks several
-       games. */
-    mycia_tac = mycia_tbc = 0;
-    mycia_tal = mycia_tbl = 0xffff;
-#else
-    /* This is according to FrodoSC, breaks Wasteland/T'Pau. */
     mycia_tac = mycia_tbc = 0xffff;
-    mycia_tal = mycia_tbl = 0x1;
-#endif
+    mycia_tal = mycia_tbl = 0xffff;
 
     mycia_tas = CIAT_STOPPED;
     mycia_tbs = CIAT_STOPPED;
     mycia_tat = 0;
     mycia_tbt = 0;
 
-    maincpu_unset_alarm(A_MYCIATB);
-    maincpu_unset_alarm(A_MYCIATA);
+    my_unset_tbi();
+    my_unset_tai();
 
     memset(myciatodalarm, 0, sizeof(myciatodalarm));
     myciatodlatched = 0;
     myciatodstopped = 0;
-    maincpu_set_alarm(A_MYCIATOD, myciatodticks);
+    mycpu_set_alarm(A_MYCIATOD, myciatodticks);
 
     myciaint = 0;
+
+    RESET_CIA
 }
 
 
 void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 {
     CLOCK rclk;
-    
+
     addr &= 0xf;
 
-    vic_ii_handle_pending_alarms(maincpu_num_write_cycles());
+    vic_ii_handle_pending_alarms(mycpu_num_write_cycles());
 
     rclk = clk - STORE_OFFSET;
 
 #ifdef MYCIA_TIMER_DEBUG
-    if (app_resources.debugFlag)
-	printf("store mycia[%02x] %02x @ clk=%d, pc=%04x\n",
-	       (int) addr, (int) byte, rclk, *reg_pcp);
+    if (mycia_debugFlag)
+	printf("store mycia[%02x] %02x @ clk=%d, pc=\n",
+	       (int) addr, (int) byte, rclk);
 #endif
 
     switch (addr) {
@@ -378,7 +443,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
       case CIA_DDRA:
 	STORE_CIAPA
 	break;
-	
+
       case CIA_PRB:		/* port B */
       case CIA_DDRB:
 	if ((mycia[CIA_CRA] | mycia[CIA_CRB]) & 0x02) {
@@ -397,23 +462,54 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	STORE_CIAPB
 	break;
 
+	/* This handles the timer latches.  The kludgy stuff is an attempt
+           emulate the correct behavior when the latch is written to during
+           an underflow.  */
       case CIA_TAL:
-	update_mycia(rclk);
-	mycia_tal = (mycia_tal & 0xff00) | byte;
+	update_tai(rclk); /* schedule alarm in case latch value is changed */
+	update_mycia(rclk - 1);
+	if (mycia_tac == mycia_tal && mycia_tas == CIAT_RUNNING) {
+	    mycia_tac = mycia_tal = (mycia_tal & 0xff00) | byte;
+	    mycia_tau = rclk + mycia_tac;
+	    update_mycia(rclk);
+	} else {
+	    mycia_tal = (mycia_tal & 0xff00) | byte;
+	}
+	break;
+      case CIA_TBL:
+	update_tbi(rclk); /* schedule alarm in case latch value is changed */
+	update_mycia(rclk - 1);
+	if (mycia_tbc == mycia_tbl && mycia_tbs == CIAT_RUNNING) {
+	    mycia_tbc = mycia_tbl = (mycia_tbl & 0xff00) | byte;
+	    mycia_tbu = rclk + mycia_tbc + 1;
+	    update_mycia(rclk);
+	} else {
+	    mycia_tbl = (mycia_tbl & 0xff00) | byte;
+	}
 	break;
       case CIA_TAH:
-	update_mycia(rclk);
-	mycia_tal = (mycia_tal & 0x00ff) | (byte << 8);
+	update_tai(rclk); /* schedule alarm in case latch value is changed */
+	update_mycia(rclk - 1);
+	if (mycia_tac == mycia_tal && mycia_tas == CIAT_RUNNING) {
+	    mycia_tac = mycia_tal = (mycia_tal & 0x00ff) | (byte << 8);
+	    mycia_tau = rclk + mycia_tac;
+	    update_mycia(rclk);
+	} else {
+	    mycia_tal = (mycia_tal & 0x00ff) | (byte << 8);
+	}
 	if (mycia_tas == CIAT_STOPPED)
 	    mycia_tac = mycia_tal;
 	break;
-      case CIA_TBL:
-	update_mycia(rclk);
-	mycia_tbl = (mycia_tbl & 0xff00) | byte;
-	break;
       case CIA_TBH:
-	update_mycia(rclk);
-	mycia_tbl = (mycia_tbl & 0x00ff) | (byte << 8);
+	update_tbi(rclk); /* schedule alarm in case latch value is changed */
+	update_mycia(rclk - 1);
+	if (mycia_tbc == mycia_tbl && mycia_tbs == CIAT_RUNNING) {
+	    mycia_tbc = mycia_tbl = (mycia_tbl & 0x00ff) | (byte << 8);
+	    mycia_tbu = rclk + mycia_tbc + 1;
+	    update_mycia(rclk);
+	} else {
+	    mycia_tbl = (mycia_tbl & 0x00ff) | (byte << 8);
+	}
 	if (mycia_tbs == CIAT_STOPPED)
 	    mycia_tbc = mycia_tbl;
 	break;
@@ -447,19 +543,26 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 
       case CIA_SDR:		/* Serial Port output buffer */
 	mycia[addr] = byte;
-	if (((mycia[CIA_CRA] & 0x40) == 0x40) && (myciasr_bits < 16)) {
-	    myciasr_bits += 16;
+	if ((mycia[CIA_CRA] & 0x40) == 0x40) {
+	    if (myciasr_bits <= 16) {
+		if(!myciasr_bits) {
+    	            STORE_SDR
+		}
+		if(myciasr_bits < 16) {
+	            /* switch timer A alarm on again, if necessary */
+	            update_mycia(rclk);
+	            if (mycia_tau) {
+		        my_set_tai_clk(mycia_tau + 1);
+	            }
+		}
 
-	    /* switch timer A alarm on again, if necessary */
-	    update_mycia(rclk);
-	    if (mycia_tau) {
-		maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
-	    }
+	        myciasr_bits += 16;
 
 #if defined (MYCIA_TIMER_DEBUG)
-	    if (app_resources.debugFlag)
-		printf("MYCIA: start SDR rclk=%d\n", rclk);
+	        if (mycia_debugFlag)
+	    	    printf("MYCIA: start SDR rclk=%d\n", rclk);
 #endif
+  	    }
 	}
 	break;
 
@@ -469,7 +572,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	update_mycia(rclk);
 
 #if defined (MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
+	if (mycia_debugFlag)
 	    printf("MYCIA set CIA_ICR: 0x%x\n", byte);
 #endif
 
@@ -481,7 +584,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 
 	/* This must actually be delayed one cycle! */
 #if defined(MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
+	if (mycia_debugFlag)
 	    printf("    set icr: ifr & ier & 0x7f -> %02x, int=%02x\n",
 		   myciaier & myciaint & 0x7f, myciaint);
 #endif
@@ -490,21 +593,22 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	}
 	if (myciaier & (CIA_IM_TA + CIA_IM_TB)) {
 	    if ((myciaier & CIA_IM_TA) && mycia_tau) {
-		maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
+		my_set_tai_clk(mycia_tau + 1);
 	    }
 	    if ((myciaier & CIA_IM_TB) && mycia_tbu) {
-		maincpu_set_alarm_clk(A_MYCIATB, mycia_tbu + 1);
+		my_set_tbi_clk(mycia_tbu + 1);
 	    }
 	}
 	/* Control */
 	break;
 
       case CIA_CRA:		/* control register A */
+	update_tai(rclk); /* schedule alarm in case latch value is changed */
 	update_mycia(rclk);
 #if defined (MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
-	    printf("MYCIA set CIA_CRA: 0x%x (clk=%d, pc=%04x, tal=%u, tac=%u)\n",
-		   byte, rclk, program_counter, mycia_tal, mycia_tac);
+	if (mycia_debugFlag)
+	    printf("MYCIA set CIA_CRA: 0x%x (clk=%d, pc=, tal=%u, tac=%u)\n",
+		   byte, rclk, /*program_counter,*/ mycia_tal, mycia_tac);
 #endif
 
 	/* bit 7 tod frequency */
@@ -515,7 +619,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	    mycia_tac = mycia_tal;
 	    if (mycia_tas == CIAT_RUNNING) {
 		mycia_tau = rclk + mycia_tac + 2;
-		maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
+		my_set_tai_clk(mycia_tau + 1);
 	    }
 	}
 	/* bit 3 timer run mode */
@@ -529,7 +633,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	    if ((byte & 0x21) == 0x01) {	/* timer just started */
 		mycia_tas = CIAT_RUNNING;
 		mycia_tau = rclk + (mycia_tac + 1) + ((byte & 0x10) >> 4);
-		maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
+		my_set_tai_clk(mycia_tau + 1);
 	    } else {		/* timer just stopped */
 		mycia_tas = CIAT_STOPPED;
 		mycia_tau = 0;
@@ -538,16 +642,14 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 		    /* 1 cycle delay for counter stop.  This must only happen
                        if we are not forcing load at the same time (i.e. bit
                        4 in `byte' is zero). */
-		    mycia_tac--;
-		    /* This should not happen, but anyway... */
-		    if (mycia_tac < 0)
-			mycia_tac = 0;
+		    if (mycia_tac > 0)
+			mycia_tac--;
 		}
-		maincpu_unset_alarm(A_MYCIATA);
+		my_unset_tai();
 	    }
 	}
 #if defined (MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
+	if (mycia_debugFlag)
 	    printf("    -> tas=%d, tau=%d\n", mycia_tas, mycia_tau);
 #endif
 	mycia[addr] = byte & 0xef;	/* remove strobe */
@@ -555,12 +657,13 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	break;
 
       case CIA_CRB:		/* control register B */
+	update_tbi(rclk); /* schedule alarm in case latch value is changed */
 	update_mycia(rclk);
 
 #if defined (MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
-	    printf("MYCIA set CIA_CRB: 0x%x (clk=%d, pc=%04x, tbl=%u, tbc=%u)\n",
-		   byte, rclk, *reg_pcp, mycia_tbl, mycia_tbc);
+	if (mycia_debugFlag)
+	    printf("MYCIA set CIA_CRB: 0x%x (clk=%d, pc=, tbl=%u, tbc=%u)\n",
+		   byte, rclk, mycia_tbl, mycia_tbc);
 #endif
 
 
@@ -571,10 +674,10 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	    if (mycia_tbs == CIAT_RUNNING) {
 		mycia_tbu = rclk + mycia_tbc + 2;
 #if defined(MYCIA_TIMER_DEBUG)
-		if (app_resources.debugFlag)
+		if (mycia_debugFlag)
 		    printf("MYCIA: rclk=%d force load: set tbu alarm to %d\n", rclk, mycia_tbu);
 #endif
-		maincpu_set_alarm_clk(A_MYCIATB, mycia_tbu + 1);
+		my_set_tbi_clk(mycia_tbu + 1);
 	    }
 	}
 	/* bit 3 timer run mode */
@@ -588,26 +691,24 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	    if ((byte & 0x61) == 0x01) {	/* timer just started */
 		mycia_tbu = rclk + (mycia_tbc + 1) + ((byte & 0x10) >> 4);
 #if defined(MYCIA_TIMER_DEBUG)
-		if (app_resources.debugFlag)
+		if (mycia_debugFlag)
 		    printf("MYCIA: rclk=%d start timer: set tbu alarm to %d\n", rclk, mycia_tbu);
 #endif
-		maincpu_set_alarm_clk(A_MYCIATB, mycia_tbu + 1);
+		my_set_tbi_clk(mycia_tbu + 1);
 		mycia_tbs = CIAT_RUNNING;
 	    } else {		/* timer just stopped */
 #if defined(MYCIA_TIMER_DEBUG)
-		if (app_resources.debugFlag)
+		if (mycia_debugFlag)
 		    printf("MYCIA: rclk=%d stop timer: set tbu alarm\n", rclk);
 #endif
-		maincpu_unset_alarm(A_MYCIATB);
+		my_unset_tbi();
 		mycia_tbu = 0;
 		if (!(byte & 0x10)) {
 		    /* 1 cycle delay for counter stop.  This must only happen
                        if we are not forcing load at the same time (i.e. bit
                        4 in `byte' is zero). */
-		    mycia_tbc--;
-		    /* This should not happen, but anyway... */
-		    if (mycia_tbc < 0)
-			mycia_tbc = 0;
+		    if (mycia_tbc > 0)
+			mycia_tbc--;
 		}
 		/* this should actually read (byte & 0x61), but as CNT is high
 		   by default, bit 0x20 is a `don't care' bit */
@@ -616,7 +717,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 		    update_mycia(rclk);
 		    /* switch timer A alarm on if necessary */
 		    if (mycia_tau) {
-			maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
+			my_set_tai_clk(mycia_tau + 1);
 		    }
 		} else {
 		    mycia_tbs = CIAT_STOPPED;
@@ -625,7 +726,7 @@ void REGPARM2 store_mycia(ADDRESS addr, BYTE byte)
 	}
 	mycia[addr] = byte & 0xef;	/* remove strobe */
 	break;
-	
+
       default:
 	mycia[addr] = byte;
     }				/* switch */
@@ -641,10 +742,10 @@ BYTE REGPARM1 read_mycia(ADDRESS addr)
 
     BYTE read_mycia_(ADDRESS addr);
     BYTE tmp = read_mycia_(addr);
-    
-    if (app_resources.debugFlag)
-	printf("read mycia[%x] returns %02x @ clk=%d, pc=%04x\n",
-	       addr, tmp, clk - READ_OFFSET, *reg_pcp);
+
+    if (mycia_debugFlag)
+	printf("read mycia[%x] returns %02x @ clk=%d, pc=\n",
+	       addr, tmp, clk - READ_OFFSET);
     return tmp;
 }
 
@@ -655,7 +756,7 @@ BYTE read_mycia_(ADDRESS addr)
 
     static BYTE byte;
     CLOCK rclk;
-    
+
     addr &= 0xf;
 
     vic_ii_handle_pending_alarms(0);
@@ -672,7 +773,7 @@ BYTE read_mycia_(ADDRESS addr)
 
       case CIA_PRB:		/* port B */
 	READ_CIAPB
-	if ((mycia[CIA_CRA] | mycia[CIA_CRB]) & 0x02) {
+        if ((mycia[CIA_CRA] | mycia[CIA_CRB]) & 0x02) {
 	    update_mycia(rclk);
 	    if (mycia[CIA_CRA] & 0x02) {
 		byte &= 0xbf;
@@ -685,6 +786,7 @@ BYTE read_mycia_(ADDRESS addr)
 		    byte |= 0x80;
 	    }
 	}
+
 	return byte;
 	break;
 
@@ -729,9 +831,9 @@ BYTE read_mycia_(ADDRESS addr)
 	/* little hack .... */
 	{
 	    int i;
-	    printf("\nmaincpu_ints:");
+	    printf("\nmycpu_ints:");
 	    for (i = 0; i < NUMOFINT; i++) {
-		printf(" %d", maincpu_int_status.pending_int[i]);
+		printf(" %d", mycpu_int_status.pending_int[i]);
 	    }
 	    printf("\n");
 	}
@@ -744,17 +846,38 @@ BYTE read_mycia_(ADDRESS addr)
 	{
 	    BYTE t = 0;
 
+	    READ_CIAICR
+#ifdef MYCIA_TIMER_DEBUG
+	    if (mycia_debugFlag)
+		printf("MYCIA read intfl: rclk=%d, alarm_ta=%d, alarm_tb=%d\n",
+			rclk, mycpu_int_status.alarm_clk[A_MYCIATA],
+			mycpu_int_status.alarm_clk[A_MYCIATB]);
+#endif
+	
 	    myciardi = rclk;
-	    update_mycia(rclk);
-	    t = myciaint;
+            t = myciaint;	/* we clean myciaint anyway, so make int_* */
+	    myciaint = 0;	/* believe it is already */
 
-#ifdef DEBUG
-	    if (app_resources.debugFlag)
+            if (rclk >= mycia_tai)
+                int_myciata(rclk - mycia_tai);
+            if (rclk >= mycia_tbi)
+                int_myciatb(rclk - mycia_tbi);
+
+	    myciaint |= t;	/* some bits can be set -> or with old value */
+
+	    update_mycia(rclk);
+	    t = myciaint | myciaflag;
+
+#ifdef MYCIA_TIMER_DEBUG
+	    if (mycia_debugFlag)
 		printf("MYCIA read intfl gives myciaint=%02x -> %02x @"
-		       " PC = %04x, sr_bits=%d, clk=%d, ta=%d, tb=%d\n",
-		       myciaint, t, PC, myciasr_bits, clk, readta(), readtb());
+		       " PC=, sr_bits=%d, clk=%d, ta=%d, tb=%d\n",
+		       myciaint, t, myciasr_bits, clk, 
+			(mycia_tac ? mycia_tac : mycia_tal),
+			mycia_tbc);
 #endif
 
+	    myciaflag = 0;
 	    myciaint = 0;
 	    my_set_int(I_MYCIAFL, 0, rclk);
 
@@ -765,6 +888,87 @@ BYTE read_mycia_(ADDRESS addr)
     return (mycia[addr]);
 }
 
+BYTE REGPARM1 peek_mycia(ADDRESS addr)
+{
+    /* This code assumes that update_mycia is a projector - called at
+     * the same cycle again it doesn't change anything. This way
+     * it does not matter if we call it from peek first in the monitor
+     * and probably the same cycle again when the CPU runs on...
+     */
+    CLOCK rclk;
+
+    addr &= 0xf;
+
+    vic_ii_handle_pending_alarms(0);
+
+    rclk = clk - READ_OFFSET;
+
+    switch (addr) {
+
+	/*
+	 * TOD clock is latched by reading Hours, and released
+	 * upon reading Tenths of Seconds. The counter itself
+	 * keeps ticking all the time.
+	 * Also note that this latching is different from the input one.
+	 */
+      case CIA_TOD_TEN:	/* Time Of Day clock 1/10 s */
+      case CIA_TOD_SEC:	/* Time Of Day clock sec */
+      case CIA_TOD_MIN:	/* Time Of Day clock min */
+      case CIA_TOD_HR:		/* Time Of Day clock hour */
+	if (!myciatodlatched)
+	    memcpy(myciatodlatch, mycia + CIA_TOD_TEN, sizeof(myciatodlatch));
+	return mycia[addr];
+
+	/* Interrupts */
+
+      case CIA_ICR:		/* Interrupt Flag Register */
+	{
+	    BYTE t = 0;
+
+	    READ_CIAICR
+#ifdef MYCIA_TIMER_DEBUG
+	    if (mycia_debugFlag)
+		printf("MYCIA read intfl: rclk=%d, alarm_ta=%d, alarm_tb=%d\n",
+			rclk, mycpu_int_status.alarm_clk[A_MYCIATA],
+			mycpu_int_status.alarm_clk[A_MYCIATB]);
+#endif
+	
+	    myciardi = rclk;
+            t = myciaint;	/* we clean myciaint anyway, so make int_* */
+	    myciaint = 0;	/* believe it is already */
+
+            if (rclk >= mycia_tai)
+                int_myciata(rclk - mycia_tai);
+            if (rclk >= mycia_tbi)
+                int_myciatb(rclk - mycia_tbi);
+
+	    myciaint |= t;	/* some bits can be set -> or with old value */
+
+	    update_mycia(rclk);
+	    t = myciaint | myciaflag;
+
+#ifdef MYCIA_TIMER_DEBUG
+	    if (mycia_debugFlag)
+		printf("MYCIA read intfl gives myciaint=%02x -> %02x @"
+		       " PC=, sr_bits=%d, clk=%d, ta=%d, tb=%d\n",
+		       myciaint, t, myciasr_bits, clk, 
+			(mycia_tac ? mycia_tac : mycia_tal),
+			mycia_tbc);
+#endif
+
+/*
+	    myciaflag = 0;
+	    myciaint = 0;
+	    my_set_int(I_MYCIAFL, 0, rclk);
+*/
+	    return (t);
+	}
+      default:
+	break;
+    }				/* switch */
+
+    return read_mycia(addr);
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -773,7 +977,7 @@ int int_myciata(long offset)
     CLOCK rclk = clk - offset;
 
 #if defined(MYCIA_TIMER_DEBUG)
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("MYCIA: int_myciata(rclk = %u, tal = %u, cra=%02x\n",
 	       rclk, mycia_tal, mycia[CIA_CRA]);
 #endif
@@ -783,12 +987,21 @@ int int_myciata(long offset)
     if ((mycia_tas == CIAT_RUNNING) && !(mycia[CIA_CRA] & 8)) {
 	/* if we do not need alarm, no PB6, no shift register, and not timer B
 	   counting timer A, then we can savely skip alarms... */
-	if ((myciaier & CIA_IM_TA)
+	if ( ( (myciaier & CIA_IM_TA) && 
+		(!(myciaint & 0x80)) )
 	    || (mycia[CIA_CRA] & 0x42)
 	    || (mycia_tbs == CIAT_COUNTTA)) {
-	    maincpu_set_alarm_clk(A_MYCIATA, rclk + mycia_tal + 1 /*+ 1*/);
+	    if(offset > mycia_tal+1) {
+	        my_set_tai_clk(
+			clk - (offset % (mycia_tal+1)) + mycia_tal + 1 );
+	    } else {
+	        my_set_tai_clk(rclk + mycia_tal + 1 );
+	    }
 	} else {
-	    maincpu_unset_alarm(A_MYCIATA);
+	    /* mycia_tai = rclk + mycia_tal +1; - now keeps tai */
+	    /* printf("mycia unset alarm: clk=%d, rclk=%d, rdi=%d -> tai=%d\n",
+			clk, rclk, myciardi, mycia_tai); */
+	    mycpu_unset_alarm(A_MYCIATA);	/* do _not_ clear mycia_tai */
 	}
     } else {
 #if 0
@@ -796,19 +1009,22 @@ int int_myciata(long offset)
 	mycia[CIA_CRA] &= 0xfe;	/* clear run flag. Correct? */
 	mycia_tau = 0;
 #endif
-	maincpu_unset_alarm(A_MYCIATA);
+	my_unset_tai();
     }
 
     if (mycia[CIA_CRA] & 0x40) {
 	if (myciasr_bits) {
 #if defined(MYCIA_TIMER_DEBUG)
-	    if (app_resources.debugFlag)
+	    if (mycia_debugFlag)
 		printf("MYCIA: rclk=%d SDR: timer A underflow, bits=%d\n",
 		       rclk, myciasr_bits);
 #endif
 	    if (!(--myciasr_bits)) {
 		myciaint |= CIA_IM_SDR;
 	    }
+	    if(myciasr_bits == 16) {
+		STORE_SDR
+	    }
 	}
     }
     if (mycia_tbs == CIAT_COUNTTA) {
@@ -816,22 +1032,25 @@ int int_myciata(long offset)
 	    mycia_tbc = mycia_tbl;
 	    mycia_tbu = rclk;
 #if defined(MYCIA_TIMER_DEBUG)
-	    if (app_resources.debugFlag)
+	    if (mycia_debugFlag)
 		printf("MYCIA: timer B underflow when counting timer A occured, rclk=%d!\n", rclk);
 #endif
 	    myciaint |= CIA_IM_TB;
-	    maincpu_set_alarm_clk(A_MYCIATB, rclk);
+	    my_set_tbi_clk(rclk);
 	} else {
 	    mycia_tbc--;
 	}
     }
+
     /* CIA_IM_TA is not set here, as it can be set in update(), reset
        by reading the ICR and then set again here because of delayed
        calling of int() */
-    if (MYCIA_INT == IK_NMI || !myciardi || myciardi <= rclk - 1)
-	if ((myciaint | CIA_IM_TA) & myciaier & 0x7f) {
-	    my_set_int(I_MYCIAFL, MYCIA_INT, rclk);
-	}
+    if ((MYCIA_INT == IK_NMI && myciardi != rclk - 1)
+        || (MYCIA_INT == IK_IRQ && myciardi < rclk - 1)) {
+        if ((myciaint | CIA_IM_TA) & myciaier & 0x7f) {
+            my_set_int(I_MYCIAFL, MYCIA_INT, rclk);
+        }
+    }
 
     return 0;
 }
@@ -850,7 +1069,7 @@ int int_myciatb(long offset)
     CLOCK rclk = clk - offset;
 
 #if defined(MYCIA_TIMER_DEBUG)
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("MYCIA: timer B int_myciatb(rclk=%d, tbs=%d)\n", rclk, mycia_tbs);
 #endif
 
@@ -860,24 +1079,32 @@ int int_myciatb(long offset)
     if (mycia_tbs == CIAT_RUNNING) {
 	if (!(mycia[CIA_CRB] & 8)) {
 #if defined(MYCIA_TIMER_DEBUG)
-	    if (app_resources.debugFlag)
+	    if (mycia_debugFlag)
 		printf("MYCIA: rclk=%d myciatb: set tbu alarm to %d\n", rclk, rclk + mycia_tbl + 1);
 #endif
 	    /* if no interrupt flag we can safely skip alarms */
 	    if (myciaier & CIA_IM_TB) {
-		maincpu_set_alarm_clk(A_MYCIATB, rclk + mycia_tbl + 1 /*+ 1*/);
+		if(offset > mycia_tbl+1) {
+		    my_set_tbi_clk( 
+			clk - (offset % (mycia_tbl+1)) + mycia_tbl + 1);
+		} else {
+		    my_set_tbi_clk(rclk + mycia_tbl + 1);
+		}
 	    } else {
-		maincpu_unset_alarm(A_MYCIATB);
+		/* mycia_tbi = rclk + mycia_tbl + 1; */
+		mycpu_unset_alarm(A_MYCIATB);
 	    }
 	} else {
-/*        mycia_tbs = CIAT_STOPPED; */
-	    /*        mycia[CIA_CRB] &= 0xfe; *//* clear start bit */
-/*        mycia_tbu = 0; */
+#if 0
+	    mycia_tbs = CIAT_STOPPED;
+	    mycia[CIA_CRB] &= 0xfe; /* clear start bit */
+	    mycia_tbu = 0;
+#endif /* 0 */
 #if defined(MYCIA_TIMER_DEBUG)
-	    if (app_resources.debugFlag)
+	    if (mycia_debugFlag)
 		printf("MYCIA: rclk=%d myciatb: unset tbu alarm\n", rclk);
 #endif
-	    maincpu_unset_alarm(A_MYCIATB);
+	    my_unset_tbi();
 	}
     } else {
 	if (mycia_tbs == CIAT_COUNTTA) {
@@ -888,37 +1115,58 @@ int int_myciatb(long offset)
 	    }
 	}
 	mycia_tbu = 0;
-	maincpu_unset_alarm(A_MYCIATB);
+	my_unset_tbi();
 #if defined(MYCIA_TIMER_DEBUG)
-	if (app_resources.debugFlag)
+	if (mycia_debugFlag)
 	    printf("MYCIA: rclk=%d myciatb: unset tbu alarm\n", rclk);
 #endif
     }
 
-    if (MYCIA_INT == IK_NMI || !myciardi || myciardi <= rclk - 1)
-	if ((myciaint | CIA_IM_TB) & myciaier & 0x7f) {
-	    my_set_int(I_MYCIAFL, MYCIA_INT, rclk);
-	}
+    if ((MYCIA_INT == IK_NMI && myciardi != rclk - 1)
+        || (MYCIA_INT == IK_IRQ && myciardi < rclk - 1)) {
+        if ((myciaint | CIA_IM_TB) & myciaier & 0x7f) {
+            my_set_int(I_MYCIAFL, MYCIA_INT, rclk);
+        }
+    }
 
     return 0;
 }
 
 /* ------------------------------------------------------------------------- */
 
+void mycia_set_flag(void)
+{
+    myciaint |= CIA_IM_FLG;
+    if (mycia[CIA_ICR] & CIA_IM_FLG) {
+        my_set_int(I_MYCIAFL, MYCIA_INT, clk);
+    }
+}
+
+void mycia_set_sdr(BYTE data)
+{
+    mycia[CIA_SDR] = data;
+    myciaint |= CIA_IM_SDR;
+    if (mycia[CIA_ICR] & CIA_IM_SDR) {
+        my_set_int(I_MYCIAFL, MYCIA_INT, clk);
+    }
+}
+
+/* ------------------------------------------------------------------------- */
+
 int int_myciatod(long offset)
 {
     int t, pm;
     CLOCK rclk = clk - offset;
 
 #ifdef DEBUG
-    if (app_resources.debugFlag)
+    if (mycia_debugFlag)
 	printf("MYCIA: TOD timer event (1/10 sec tick), tod=%02x:%02x,%02x.%x\n",
 	       mycia[CIA_TOD_HR], mycia[CIA_TOD_MIN], mycia[CIA_TOD_SEC],
 	       mycia[CIA_TOD_TEN]);
 #endif
 
     /* set up new int */
-    maincpu_set_alarm(A_MYCIATOD, myciatodticks);
+    mycpu_set_alarm(A_MYCIATOD, myciatodticks);
 
     if (!myciatodstopped) {
 	/* inc timer */
@@ -945,7 +1193,7 @@ int int_myciatod(long offset)
 	    }
 	}
 #ifdef DEBUG
-	if (app_resources.debugFlag)
+	if (mycia_debugFlag)
 	    printf("MYCIA: TOD after event :tod=%02x:%02x,%02x.%x\n",
 	       mycia[CIA_TOD_HR], mycia[CIA_TOD_MIN], mycia[CIA_TOD_SEC],
 		   mycia[CIA_TOD_TEN]);
@@ -959,16 +1207,25 @@ int int_myciatod(long offset)
 /* -------------------------------------------------------------------------- */
 
 
-void mycia_prevent_clk_overflow()
+void mycia_prevent_clk_overflow(CLOCK sub)
 {
+
+    update_tai(clk);
+    update_tbi(clk);
+
     update_mycia(clk);
 
+    if(mycia_tai && (mycia_tai != -1))
+        mycia_tai -= sub;
+    if(mycia_tbi && (mycia_tbi != -1))
+        mycia_tbi -= sub;
+
     if (mycia_tau)
-	mycia_tau -= PREVENT_CLK_OVERFLOW_SUB;
+	mycia_tau -= sub;
     if (mycia_tbu)
-	mycia_tbu -= PREVENT_CLK_OVERFLOW_SUB;
-    if (myciardi > PREVENT_CLK_OVERFLOW_SUB)
-	myciardi -= PREVENT_CLK_OVERFLOW_SUB;
+	mycia_tbu -= sub;
+    if (myciardi > sub)
+	myciardi -= sub;
     else
 	myciardi = 0;
 }
@@ -1003,7 +1260,7 @@ void mycia_dump(FILE * fp)
 	);
 
     fprintf(fp, "ICR %u %u %u", myciaint, myciaier,
-	    maincpu_int_status.pending_int[I_MYCIAFL]);
+	    mycpu_int_status.pending_int[I_MYCIAFL]);
     fprintf(fp, " [fl= %s%s%s%s%s] ",
 	    (myciaint & 1) ? "TA " : "",
 	    (myciaint & 2) ? "TB " : "",
@@ -1043,7 +1300,7 @@ void mycia_undump_line(char *s)
 	if ((mycia[CIA_CRA] & 0x21) == 0x01) {
 	    mycia_tau = clk + mycia_tac;
 	    mycia_tas = CIAT_RUNNING;
-	    maincpu_set_alarm_clk(A_MYCIATA, mycia_tau + 1);
+	    my_set_tai_clk(mycia_tau + 1);
 	} else {
 	    mycia_tau = 0;
 	    mycia_tas = CIAT_STOPPED;
@@ -1054,7 +1311,7 @@ void mycia_undump_line(char *s)
 	if ((mycia[CIA_CRB] & 0x61) == 0x01) {
 	    mycia_tbu = clk + mycia_tbc;
 	    mycia_tbs = CIAT_RUNNING;
-	    maincpu_set_alarm_clk(A_MYCIATB, mycia_tbu + 1);
+	    my_set_tbi_clk(mycia_tbu + 1);
 	} else {
 	    mycia_tbu = 0;
 	    if ((mycia[CIA_CRB] & 0x61) == 0x41) {
@@ -1089,3 +1346,4 @@ void mycia_undump_line(char *s)
 #endif
 
 POST_CIA_FUNCS
+
