@@ -95,7 +95,9 @@
 
 /* Ettore Perazzoli (ettore@comm2000.it) 1997/10/27: Renamed `p_toascii' to
    `_p_toascii' to avoid conflicts with `charsets.h'. */
-   
+
+#include "vice.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -105,13 +107,14 @@
 #include <string.h>
 #endif
 
-#include "patchlevel.h"
 #include "charsets.h"		/* ctrl1, ctrl2, cbmkeys */
 
 #ifndef SEEK_SET
 #define SEEK_SET 0
 #endif
 
+#define PETCATVERSION	2.02
+#define PETCATLEVEL	1
 
 #define B_1		 1
 #define B_2		 2
@@ -347,7 +350,7 @@ char   *speechkwcc[] = {
 };
 
 
-/* @Basic (Atbasic) Keywords (Tokens CC - F6) -- Andre Fachat */
+/* @Basic (Atbasic) Keywords (Tokens CC - F6) -- Andr Fachat */
 
 char   *atbasickwcc[] = {
     "trace", "delete",  "auto", "old", "dump", "find", "renumber", "dload",
@@ -413,9 +416,10 @@ int    main(argc, argv)
     int     argc;
     char   *argv[];
 {
-    char   *progname, *outfilename = NULL, c;
+    char   *progname, *outfilename = NULL;
     char    realname[24];
     int     reclen;
+    int     c;
 
     long    offset = 0;
     int     wr_mode = 0, version = B_7;		/* best defaults */
@@ -856,7 +860,7 @@ static void   list_keywords(version)
 	    printf("%s\t", kwce[n] /*, 0xce, n*/);
     }
 
-    else 
+    else
       switch (version) {
       case B_SUPER:
 	for (n = 0; n < NUM_SECC; n++)
