@@ -19,24 +19,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
 /* This file is ugly, and needs better organization.  In particular, it would
    be nice to remove all the machine-specific hacks.  And it would be nice to
-   put it into a standalone module, although this might affect
-   performance. */
+   put it into a standalone module, although this would probably affect
+   performance.  */
 
 #define _RASTER_C
 
-#if defined( __hpux)
-#define _INCLUDE_HPUX_SOURCE
-#endif
-#include <sys/time.h>
-#if defined( __hpux)
-#undef _INCLUDE_HPUX_SOURCE
-#endif
 #include <signal.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -60,7 +54,7 @@
 #define MAX(a, b)	((a) > (b) ? (a) : (b))
 
 /* XShmPutImage is a bit more picky about its parameters than XPutImage.
-   This is useful to debug XLib failures caused by wrong calls. */
+   This is useful to debug XLib failures caused by wrong calls.  */
 /* #define RASTER_DEBUG_PUTIMAGE_CALLS */
 
 inline static void refresh_all(void);
@@ -112,15 +106,21 @@ inline static void add_ptr_change(struct changes *c, int where,
 
 inline static void add_int_change_next_line(int *ptr, int new_value)
 {
-    add_int_change(&next_line_changes, 0, ptr, new_value);
+    if (skip_next_frame)
+	*ptr = new_value;
+    else
+	add_int_change(&next_line_changes, 0, ptr, new_value);
 }
 
 inline static void add_ptr_change_next_line(void **ptr, void *new_value)
 {
-    add_ptr_change(&next_line_changes, 0, ptr, new_value);
+    if (skip_next_frame)
+	*ptr = new_value;
+    else
+	add_ptr_change(&next_line_changes, 0, ptr, new_value);
 }
 
-inline static void add_int_change_foreground(int char_x, int *ptr, 
+inline static void add_int_change_foreground(int char_x, int *ptr,
                                              int new_value)
 {
     if (skip_next_frame || char_x <= 0) {
@@ -172,10 +172,10 @@ inline static void add_ptr_change_background(int raster_x, void **ptr,
     }
 }
 
-inline static void add_int_change_border(int raster_x, int *ptr, 
+inline static void add_int_change_border(int raster_x, int *ptr,
 					 int new_value)
 {
-    if (raster_x <= 0) {
+    if (skip_next_frame || raster_x <= 0) {
 	*ptr = new_value;
     } else if (raster_x < SCREEN_WIDTH) {
 	add_int_change(&border_changes, raster_x, ptr, new_value);
@@ -193,12 +193,12 @@ inline static void add_int_change_border(int raster_x, int *ptr,
 inline static void vid_memset(PIXEL *dst, PIXEL c, int cnt)
 {
     int i;
-    
+
     for (i = 0; i < cnt; i++)
 	dst[i] = c;
 }
 
-/* This is used to paint borders and blank lines. */
+/* This is used to paint borders and blank lines.  */
 #define DRAW_BLANK(p, start, end, pixel_width)			\
     do {							\
 	PIXEL *dst;						\
@@ -212,21 +212,21 @@ inline static void vid_memset(PIXEL *dst, PIXEL c, int cnt)
 #else  /* 8 bit depth */
 #define vid_memset(dst, c, cnt) memset(dst,c,cnt)
 
-/* This is used to paint borders and blank lines. */
+/* This is used to paint borders and blank lines.  */
 #define DRAW_BLANK(p, start, end, pixel_width)		\
     vid_memset(((BYTE *)(p) + (start) * pixel_width),	\
 	       PIXEL(border_color),			\
 	       ((end) - (start) + 1) * pixel_width)
 
 #endif
-      
+
 /* ------------------------------------------------------------------------- */
 
-/* Initialization. */
+/* Initialization.  */
 static void reset_raster(void)
 {
     int i;
-    
+
     frame_buffer_ptr = (FRAME_BUFFER_START(frame_buffer)
 			+ 2 * SCREEN_MAX_SPRITE_WIDTH);
 
@@ -239,7 +239,7 @@ static void reset_raster(void)
 #ifdef __VIC_II__
     mem_counter_inc = SCREEN_TEXTCOLS;
 #endif
-    
+
     ycounter = 0;
     asleep = 0;
 
@@ -266,7 +266,7 @@ static void reset_raster(void)
 static int init_raster(int active, int max_pixel_width, int max_pixel_height)
 {
     /* Keep enough space on the left and the right for one sprite of the
-       maximum size.  This way we can clip sprites more easily. */
+       maximum size.  This way we can clip sprites more easily.  */
     if (frame_buffer_alloc(&frame_buffer,
 			   ((SCREEN_WIDTH + 2 * 2 * SCREEN_MAX_SPRITE_WIDTH)
 			    * max_pixel_width),
@@ -279,43 +279,46 @@ static int init_raster(int active, int max_pixel_width, int max_pixel_height)
 }
 
 /* Resize the canvas with the specified values and center the screen image on
-   it.  The actual size can be different if the parameters are not suitable. */
+   it.  The actual size can be different if the parameters are not suitable.  */
 static void resize(unsigned int width, unsigned int height)
 {
-    if (width > SCREEN_WIDTH * pixel_width) {
-#ifndef __MSDOS__
-	width = SCREEN_WIDTH * pixel_width;
-#endif
+    if (width >= SCREEN_WIDTH * pixel_width) {
 	window_x_offset = (width - SCREEN_WIDTH * pixel_width) / 2;
 	window_first_x = 0;
     } else {
 	window_x_offset = 0;
-	window_first_x = (SCREEN_WIDTH - width / pixel_width) / 2;
 #ifndef SCREEN_BORDERWIDTH_VARIES
- 	if (window_first_x > SCREEN_BORDERWIDTH)
+	if (width > SCREEN_XPIX * pixel_width)
+	    window_first_x = (SCREEN_BORDERWIDTH
+			      - (width / pixel_width - SCREEN_XPIX) / 2);
+	else
 	    window_first_x = SCREEN_BORDERWIDTH;
+#else
+	window_first_x = (SCREEN_WIDTH - width / pixel_width) / 2;
 #endif
     }
-    
-    if (height > SCREEN_HEIGHT * pixel_height) {
-#ifndef __MSDOS__
-	height = SCREEN_HEIGHT * pixel_height;
-#endif
+
+    if (height >= SCREEN_HEIGHT * pixel_height) {
 	window_y_offset = (height - SCREEN_HEIGHT * pixel_height) / 2;
 	window_first_line = 0;
-    } else { 
+	window_last_line = window_first_line + SCREEN_HEIGHT - 1;
+    } else {
 	window_y_offset = 0;
-	window_first_line = (SCREEN_HEIGHT - height / pixel_height) / 2;
-#ifndef SCREEN_BORDERWIDTH_VARIES
- 	if (window_first_line > SCREEN_BORDERHEIGHT)
+#ifndef SCREEN_BORDERHEIGHT_VARIES
+	if (height > SCREEN_YPIX * pixel_height)
+	    window_first_line = (SCREEN_BORDERHEIGHT
+				 - (height / pixel_height - SCREEN_YPIX) / 2);
+	else
 	    window_first_line = SCREEN_BORDERHEIGHT;
+#else
+	window_first_line = (SCREEN_HEIGHT - height / pixel_height) / 2;
 #endif
+	window_last_line = window_first_line + height / pixel_height;
     }
+
     canvas_resize(canvas, width, height);
 
-    window_last_line = window_first_line + height / pixel_height;
-
-    /* Make sure we don't waste space showing unused lines. */
+    /* Make sure we don't waste space showing unused lines.  */
     if ((window_first_line < SCREEN_FIRST_DISPLAYED_LINE
 	 && window_last_line < SCREEN_LAST_DISPLAYED_LINE)
 	|| (window_first_line > SCREEN_FIRST_DISPLAYED_LINE
@@ -323,14 +326,14 @@ static void resize(unsigned int width, unsigned int height)
 	window_first_line = SCREEN_FIRST_DISPLAYED_LINE;
 	window_last_line = window_first_line + height / pixel_height;
     }
-    
+
     window_width = width;
     window_height = height;
 }
 
-/* Open the emulation window. */
+/* Open the emulation window.  */
 static int open_output_window(char *win_name, unsigned int width,
-			      unsigned int height, color_def_t color_defs[],
+			      unsigned int height, palette_t *palette,
 			      canvas_redraw_t exposure_handler)
 {
     int i;
@@ -339,10 +342,9 @@ static int open_output_window(char *win_name, unsigned int width,
     window_height = height * pixel_height;
 
     canvas = canvas_create(win_name, &window_width, &window_height, !asleep,
-			   exposure_handler, SCREEN_NUM_COLORS, color_defs,
-			   pixel_table);
+			   exposure_handler, palette, pixel_table);
 
-    /* Prepare the double and quad pixel tables. */
+    /* Prepare the double and quad pixel tables.  */
     for (i = 0; i < 0x100; i++)
 	*((PIXEL *)(double_pixel_table + i))
 	    = *((PIXEL *)(double_pixel_table + i) + 1) = pixel_table[i];
@@ -363,10 +365,47 @@ static int open_output_window(char *win_name, unsigned int width,
 
 /* ------------------------------------------------------------------------- */
 
+/* set a new palette */
+
+static int set_palette_file_name(resource_value_t v)
+{
+    int i;
+    /* If called before initialization, just set the resource value.  The
+       palette file will be loaded afterwards.  */
+    if (palette == NULL) {
+        string_set(&palette_file_name, (char *) v);
+        return 0;
+    }
+
+    if (palette_load((char *) v, palette) < 0) {
+        fprintf(stderr, "Couldn't load palette `%s'\n", (char *) v);
+        return -1;
+    }
+    canvas_set_palette(canvas, palette, pixel_table);
+
+    /* Prepare the double and quad pixel tables.  */
+    for (i = 0; i < 0x100; i++)
+	*((PIXEL *)(double_pixel_table + i))
+	    = *((PIXEL *)(double_pixel_table + i) + 1) = pixel_table[i];
+    for (i = 0; i < 0x100; i++)
+	*((PIXEL2 *)(quad_pixel_table + i))
+	    = *((PIXEL2 *)(quad_pixel_table + i) + 1) = double_pixel_table[i];
+
+    init_drawing_tables();
+
+    /* Make sure the pixel tables are recalculated properly.  */
+    video_resize();
+
+    string_set(&palette_file_name, (char *) v);
+    return 0;
+}
+
+/* ------------------------------------------------------------------------- */
+
 /* Read length bytes from src and store them in dest, checking for differences
    between the two arrays.  The smallest interval that contains different bytes
-   is returned as [*xs; *xe]. */
-inline static int _fill_cache(BYTE * dest, BYTE * src, int length, int srcstep,
+   is returned as [*xs; *xe].  */
+inline static int _fill_cache(BYTE *dest, BYTE *src, int length, int srcstep,
 			      int *xs, int *xe, int no_check)
 {
     if (no_check) {
@@ -403,7 +442,7 @@ inline static int _fill_cache(BYTE * dest, BYTE * src, int length, int srcstep,
 }
 
 /* Do as _fill_cache(), but split each byte into low and high nibble.  These
-   are stored into different destinations. */
+   are stored into different destinations.  */
 inline static int _fill_cache_nibbles(BYTE * desthi, BYTE * destlo, BYTE * src,
 				      int length, int srcstep, int *xs, int *xe,
 				      int no_check)
@@ -449,7 +488,7 @@ inline static int _fill_cache_nibbles(BYTE * desthi, BYTE * destlo, BYTE * src,
 
 
 /* This function is used for text modes.  It checks for differences in the
-   character memory too. */
+   character memory too.  */
 inline static int _fill_cache_text(BYTE * dest, BYTE * src, BYTE * charmem,
 				   int length, int l, int *xs, int *xe,
 				   int no_check)
@@ -487,7 +526,7 @@ inline static int _fill_cache_text(BYTE * dest, BYTE * src, BYTE * charmem,
 
 #if SCREEN_NUM_SPRITES > 0
 /* Fill the sprite cache with the new sprite data. [*xs; *xe] is the changed
-   interval (in pixels). */
+   interval (in pixels).  */
 inline static int _fill_sprite_cache(struct line_cache *ll, int *xs, int *xe)
 {
     int rr = 0, i, r, sxe, sxs, sxe1, sxs1, n = 0, msk;
@@ -504,7 +543,7 @@ inline static int _fill_sprite_cache(struct line_cache *ll, int *xs, int *xe)
 	r = 0;
 	if (dma_msk & msk) {
 	    DWORD data = sprite_data[i];
-	    
+
 	    ll->sprmask |= msk;
 	    sxe = spr.x + (spr.x_expanded ? 48 : 24);
 	    sxs = spr.x;
@@ -549,12 +588,12 @@ inline static int _fill_sprite_cache(struct line_cache *ll, int *xs, int *xe)
 		sprl->in_background = spr.in_background;
 		r = 1;
 	    }
-	    
+
 	    if (sprl->data != data) {
 		sprl->data = data;
 		r = 1;
 	    }
-	    
+
 	    if (r) {
 		xs_return = MIN(xs_return, sxs);
 		xe_return = MAX(xe_return, sxe);
@@ -583,7 +622,7 @@ inline static int _fill_sprite_cache(struct line_cache *ll, int *xs, int *xe)
 
 /* Increase the size of the rectangle to be refreshed so that it also includes
    the interval [xs; xe] of line y.  The lines must be added in order: from
-   top to bottom. */
+   top to bottom.  */
 inline static void add_line(int y, int xs, int xe)
 {
 #ifdef RASTER_DEBUG_PUTIMAGE_CALLS
@@ -608,7 +647,7 @@ inline static void add_line(int y, int xs, int xe)
 #endif
 }
 
-/* Refresh the changed rectangle. */
+/* Refresh the changed rectangle.  */
 inline static void refresh_changed(void)
 {
     int x, y, xx, yy;
@@ -642,13 +681,13 @@ inline static void refresh_changed(void)
     y *= pixel_height;
     yy *= pixel_height;
     h *= pixel_height;
-    
+
 #if defined (RASTER_DEBUG_PUTIMAGE_CALLS)
     printf("Refresh %d %d %d %d %d %d \n", x, y, xx, yy, w, h);
 #endif
 
     x += 2 * SCREEN_MAX_SPRITE_WIDTH;
-    
+
     canvas_refresh(canvas, frame_buffer, x, y,
 		   xx + window_x_offset, yy + window_y_offset,
 		   w, h);
@@ -656,25 +695,30 @@ inline static void refresh_changed(void)
     changed_area.is_null = 1;
 }
 
-/* Unconditionally refresh the whole screen. */
+/* Unconditionally refresh the whole screen.  */
 inline static void refresh_all(void)
 {
 #if defined (RASTER_DEBUG_PUTIMAGE_CALLS)
-    printf("Global refresh %d %d %d %d %d %d\n", window_first_x * pixel_width,
-    window_first_line * pixel_height, 0, 0, window_width, window_height);
+    printf("Global refresh %d %d %d %d %d %d\n",
+	   window_first_x * pixel_width + 2 * SCREEN_MAX_SPRITE_WIDTH,
+	   window_first_line * pixel_height,
+	   window_x_offset, window_y_offset,
+	   MIN(window_width, SCREEN_WIDTH * pixel_width),
+	   MIN(window_height, SCREEN_HEIGHT * pixel_height));
 #endif
     canvas_refresh(canvas, frame_buffer,
 		   window_first_x * pixel_width + 2 * SCREEN_MAX_SPRITE_WIDTH,
 		   window_first_line * pixel_height,
 		   window_x_offset, window_y_offset,
-		   window_width, window_height);
+		   MIN(window_width, SCREEN_WIDTH * pixel_width),
+		   MIN(window_height, SCREEN_HEIGHT * pixel_height));
 }
 
 /* ------------------------------------------------------------------------- */
 
 inline static void handle_blank_line(void)
 {
-    /* changes... should be handled better. */
+    /* Changes... Should/could be handled better.  */
     if (have_changes_on_this_line) {
         int i, xs;
 
@@ -696,31 +740,36 @@ inline static void handle_blank_line(void)
 	cache[rasterline].blank = 1;
     	have_changes_on_this_line = 0;
 	add_line(rasterline, 0, SCREEN_WIDTH - 1);
-	if (pixel_height == 2 && app_resources.doubleScan)
+	if (pixel_height == 2 && double_scan_enabled)
 	    vid_memcpy((FRAME_BUFFER_LINE_START(frame_buffer, 2*rasterline + 1)
 			+ 2 * SCREEN_MAX_SPRITE_WIDTH),
 	               frame_buffer_ptr, pixel_width * SCREEN_WIDTH);
-    } else if (dont_cache
+    } else if (CANVAS_USES_TRIPLE_BUFFERING(canvas)
+               || dont_cache
 	       || cache[rasterline].is_dirty
 	       || border_color != cache[rasterline].border_color
 	       || !cache[rasterline].blank) {
-	
+
         /* Even when the actual caching is disabled, redraw blank lines only if
-           it is really necessary to do so. */
+           it is really necessary to do so.  */
 	cache[rasterline].border_color = border_color;
 	cache[rasterline].blank = 1;
 	cache[rasterline].is_dirty = 0;
-	
+
 	DRAW_BLANK(frame_buffer_ptr, 0, SCREEN_WIDTH - 1, pixel_width);
-	if (pixel_height == 2 && app_resources.doubleScan)
+	if (pixel_height == 2 && double_scan_enabled)
 	    DRAW_BLANK((FRAME_BUFFER_LINE_START(frame_buffer, 2*rasterline + 1)
 			+ 2 * SCREEN_MAX_SPRITE_WIDTH),
 		       0, SCREEN_WIDTH - 1, pixel_width);
 	add_line(rasterline, 0, SCREEN_WIDTH - 1);
     }
+
+#ifdef __VIC_II__
+    update_sprite_collisions();
+#endif
 }
 
-/* Draw the borders. */
+/* Draw the borders.  */
 inline static void draw_borders(void)
 {
     if (!open_left_border)
@@ -740,16 +789,21 @@ inline static void handle_visible_line_with_cache(void)
 
     line = rasterline - SCREEN_BORDERHEIGHT - ysmooth - 1;
 
-    /* Check for `major' changes first.  If there is any, just write straight
-       to the cache without any comparisons and redraw the whole line. */
-    if (l->is_dirty || dont_cache || l->n != line || l->xsmooth != xsmooth
-	|| l->video_mode != vm || l->blank || l->ycounter != ycounter
+    /* Check for "major" changes first.  If there is any, just write straight
+       to the cache without any comparisons and redraw the whole line.  */
+    if (l->is_dirty
+	|| dont_cache
+	|| l->n != line
+	|| l->xsmooth != xsmooth
+	|| l->video_mode != vm
+	|| l->blank
+	|| l->ycounter != ycounter
 	|| l->border_color != border_color
 	|| l->display_xstart != display_xstart
 	|| l->display_xstop != display_xstop
 	|| (l->open_right_border && !open_right_border)
         || (l->open_left_border && !open_left_border)) {
-	
+
 	needs_update = 1;
 	l->n = line;
 	l->video_mode = vm;
@@ -762,21 +816,25 @@ inline static void handle_visible_line_with_cache(void)
 	l->open_right_border = open_right_border;
 	l->open_left_border = open_left_border;
 
+#if defined(__VIC_II__)
+	l->overscan_background_color = overscan_background_color;
+#endif /* defined(__VIC_II__) */
+
 	/* Fill the space between the border and the graphics with the
-	   background color (necessary if `xsmooth' is != 0). */
+	   background color (necessary if `xsmooth' is != 0).  */
 	vid_memset(frame_buffer_ptr + SCREEN_BORDERWIDTH * pixel_width,
-		   PIXEL(background_color), xsmooth * pixel_width);
-	
+		   PIXEL(overscan_background_color), xsmooth * pixel_width);
+
 	if (open_left_border)
-	    vid_memset(frame_buffer_ptr, PIXEL(background_color),
+	    vid_memset(frame_buffer_ptr, PIXEL(overscan_background_color),
 		       (SCREEN_BORDERWIDTH + xsmooth) * pixel_width);
 	if (open_right_border)
 	    vid_memset((frame_buffer_ptr +
 			((SCREEN_BORDERWIDTH + SCREEN_XPIX + xsmooth)
-			 * pixel_width)), PIXEL(background_color),
+			 * pixel_width)), PIXEL(overscan_background_color),
 		       (SCREEN_WIDTH - SCREEN_BORDERWIDTH - SCREEN_XPIX
 			- xsmooth) * pixel_width);
-    
+
 #if SCREEN_NUM_SPRITES > 0
 	_fill_sprite_cache(l, &changed_start, &changed_end);
 #endif
@@ -785,17 +843,17 @@ inline static void handle_visible_line_with_cache(void)
 
 	/* [ `changed_start' ; `changed_end' ] now covers the whole line, as
 	   we have called fill_cache() with `1' as the last parameter (no
-	   check). */
+	   check).  */
 	video_modes[vm].draw_line_cached(l, changed_start, changed_end);
 
 	changed_start = 0;
 	changed_end = SCREEN_WIDTH - 1;
 
 	draw_borders();
-	
+
     } else {
 
-	/* There are no `major' changes: try to do some optimization. */
+	/* There are no `major' changes: try to do some optimization.  */
 
 #if SCREEN_NUM_SPRITES > 0
 
@@ -805,51 +863,81 @@ inline static void handle_visible_line_with_cache(void)
 	sprites_need_update = _fill_sprite_cache(l, &sprite_changed_start,
 						 &sprite_changed_end);
 	/* If sprites have changed, do not bother trying to reduce the amount
-	   of recalculated data, but simply redraw everything. */
+	   of recalculated data, but simply redraw everything.  */
 	needs_update = video_modes[vm].fill_cache(l, &changed_start,
 						  &changed_end,
 						  sprites_need_update);
 
+#if defined(__VIC_II__)
+	/* If the background color changes, we might get the wrong color in
+           the left part of the screen, between the border and the start of
+           the graphics.  */
+	if (ysmooth > 0
+	    && l->overscan_background_color != overscan_background_color)
+	    needs_update = 1;
+#endif /* defined(__VIC_II__) */
+
 	if (needs_update) {
 	    video_modes[vm].draw_line_cached(l, changed_start, changed_end);
 
 	    /* Fill the space between the border and the graphics with the
-	       background color (necessary if xsmooth is > 0). */
+	       background color (necessary if xsmooth is > 0).  */
 	    vid_memset(frame_buffer_ptr + SCREEN_BORDERWIDTH * pixel_width,
-		       PIXEL(background_color), xsmooth * pixel_width);
+		       PIXEL(overscan_background_color), xsmooth * pixel_width);
 
 	    /* If xsmooth > 0, drawing the graphics might have corrupted
-	       part of the border... fix it here. */
+	       part of the border... fix it here.  */
 	    if (!open_right_border)
 		DRAW_BLANK(frame_buffer_ptr, SCREEN_BORDERWIDTH + SCREEN_XPIX,
 			   SCREEN_BORDERWIDTH + SCREEN_XPIX + 8, pixel_width);
 
-	    /* Convert from character to pixel coordinates. */
+	    /* Calculate the interval in pixel coordinates.  */
+
+#if defined(__VIC_II__)
+	    if (l->overscan_background_color != overscan_background_color) {
+		if (ysmooth > 0)
+		    changed_start = SCREEN_BORDERWIDTH;
+		else
+		    changed_start = (SCREEN_BORDERWIDTH + xsmooth
+				     + 8 * changed_start);
+		l->overscan_background_color = overscan_background_color;
+	    } else {
+		changed_start = (SCREEN_BORDERWIDTH + xsmooth
+				 + 8 * changed_start);
+	    }
+#else  /* defined(__VIC_II__) */
 	    changed_start = SCREEN_BORDERWIDTH + xsmooth + 8 * changed_start;
+#endif /* defined(__VIC_II__) */
+
 	    changed_end = (SCREEN_BORDERWIDTH + xsmooth + 8 * (changed_end + 1)
 			   - 1);
-	    
-	    if (sprites_need_update) {
 
-		/* FIXME: wrong. */
+	    if (sprites_need_update) {
+		/* FIXME: wrong.  */
 		if (open_left_border)
 		    changed_start = 0;
 		if (open_right_border)
 		    changed_end = SCREEN_WIDTH - 1;
-		
+
 		/* Even if we have recalculated the whole line, we will refresh
 		   only the part that has actually changed when writing to the
-		   window. */
+		   window.  */
 		changed_start = MIN(changed_start, sprite_changed_start);
 		changed_end = MAX(changed_end, sprite_changed_end);
 
 		/* The borders have not changed, so do not repaint them even
-		   if there are sprites under them. */
+		   if there are sprites under them.  */
 		changed_start = MAX(changed_start, display_xstart);
 		changed_end = MIN(changed_end, display_xstop);
 	    }
 	}
-	
+#if SCREEN_NUM_SPRITES > 0
+        else {                /* if (needs_update) */
+            cl_ss_collmask = l->ss_collmask;
+            cl_sb_collmask = l->sb_collmask;
+        }
+#endif
+
 #else  /* no sprites */
 
 	needs_update = video_modes[vm].fill_cache(l, &changed_start,
@@ -858,7 +946,7 @@ inline static void handle_visible_line_with_cache(void)
 	if (needs_update) {
 	    video_modes[vm].draw_line_cached(l, changed_start, changed_end);
 
-	    /* Convert from character to pixel coordinates. */
+	    /* Convert from character to pixel coordinates.  */
 	    changed_start = SCREEN_BORDERWIDTH + xsmooth + 8 * changed_start;
 	    changed_end = (SCREEN_BORDERWIDTH + xsmooth + 8 * (changed_end + 1)
 			   - 1);
@@ -871,7 +959,7 @@ inline static void handle_visible_line_with_cache(void)
     if (needs_update) {
 	add_line(rasterline, changed_start, changed_end);
 
-	if (pixel_height == 2 && app_resources.doubleScan) {
+	if (pixel_height == 2 && double_scan_enabled) {
 	    vid_memcpy((FRAME_BUFFER_LINE_START(frame_buffer, 2*rasterline + 1)
 		        + 2*SCREEN_MAX_SPRITE_WIDTH
 			+ changed_start * pixel_width),
@@ -885,22 +973,22 @@ inline static void handle_visible_line_with_cache(void)
 
 inline static void handle_visible_line_without_cache()
 {
-    /* If screen is scrolled to the right, we need to fill with the background 
-       color the blank part on the left. */
+    /* If screen is scrolled to the right, we need to fill with the background
+       color the blank part on the left.  */
     vid_memset(frame_buffer_ptr + SCREEN_BORDERWIDTH * pixel_width,
-	       PIXEL(background_color), xsmooth * pixel_width);
+	       PIXEL(overscan_background_color), xsmooth * pixel_width);
 
     if (open_left_border)
-	vid_memset(frame_buffer_ptr, PIXEL(background_color),
+	vid_memset(frame_buffer_ptr, PIXEL(overscan_background_color),
 		   (SCREEN_BORDERWIDTH + xsmooth) * pixel_width);
     if (open_right_border)
 	vid_memset((frame_buffer_ptr +
 		    ((SCREEN_BORDERWIDTH + SCREEN_XPIX + xsmooth)
-		     * pixel_width)), PIXEL(background_color),
+		     * pixel_width)), PIXEL(overscan_background_color),
 		    (SCREEN_WIDTH - SCREEN_BORDERWIDTH - SCREEN_XPIX
 		     - xsmooth) * pixel_width);
-    
-    /* Draw the graphics and sprites. */
+
+    /* Draw the graphics and sprites.  */
     if (draw_idle_state)
 	video_modes[SCREEN_IDLE_MODE].draw_line();
     else
@@ -908,39 +996,44 @@ inline static void handle_visible_line_without_cache()
 
     draw_borders();
 
-    /* Still do some minimal caching anyway. */
-    if (dont_cache
+    if (CANVAS_USES_TRIPLE_BUFFERING(canvas)
+        || dont_cache
+	|| dma_msk
 	|| cache[rasterline].is_dirty
 	|| cache[rasterline].blank
-	|| border_color != cache[rasterline].border_color
-	|| dma_msk
-	|| (cache[rasterline].open_right_border && !open_right_border)
-	|| (cache[rasterline].open_left_border && !open_left_border)) {
-
+	|| cache[rasterline].border_color != border_color
+	|| (cache[rasterline].open_right_border != open_right_border)
+	|| (cache[rasterline].open_left_border != open_left_border)
+#if defined(__VIC_II__)
+	|| (cache[rasterline].overscan_background_color
+	    != overscan_background_color)
+#endif
+	) {
 	cache[rasterline].blank = 0;
+	cache[rasterline].is_dirty = 0;
 	cache[rasterline].border_color = border_color;
 	cache[rasterline].open_right_border = open_right_border;
 	cache[rasterline].open_left_border = open_left_border;
-	cache[rasterline].is_dirty = 0;
-	
+#if defined(__VIC_II__)
+	cache[rasterline].overscan_background_color = overscan_background_color;
+#endif
 	add_line(rasterline, 0, SCREEN_WIDTH - 1);
 
-	if (pixel_height == 2 && app_resources.doubleScan)
-	    vid_memcpy(FRAME_BUFFER_LINE_START(frame_buffer, rasterline*2 + 1)
-		       + 2 * SCREEN_MAX_SPRITE_WIDTH,
-		       frame_buffer_ptr, SCREEN_WIDTH * pixel_width);
     } else {
 
-	add_line(rasterline, SCREEN_BORDERWIDTH,
-		 SCREEN_BORDERWIDTH + SCREEN_XPIX);
-	if (pixel_height == 2 && app_resources.doubleScan)
-	    vid_memcpy((FRAME_BUFFER_LINE_START(frame_buffer, rasterline*2 + 1)
-			+ 2 * SCREEN_MAX_SPRITE_WIDTH
-			+ SCREEN_BORDERWIDTH * pixel_width),
-		       frame_buffer_ptr + SCREEN_BORDERWIDTH * pixel_width,
-		       SCREEN_XPIX * pixel_width);
-	
+        /* Still do some minimal caching anyway.  */
+	/* Only update the part between the borders.  */
+	add_line(rasterline,
+		 SCREEN_BORDERWIDTH,
+		 SCREEN_BORDERWIDTH + SCREEN_XPIX - 1);
+
     }
+
+    if (pixel_height == 2 && double_scan_enabled)
+	vid_memcpy((FRAME_BUFFER_LINE_START(frame_buffer, rasterline*2 + 1)
+		    + 2 * SCREEN_MAX_SPRITE_WIDTH),
+		   frame_buffer_ptr,
+		   SCREEN_WIDTH * pixel_width);
 }
 
 inline static void handle_visible_line_with_changes(void)
@@ -952,13 +1045,15 @@ inline static void handle_visible_line_with_changes(void)
 
         if (xs < xe) {
             int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+
 	    video_modes[vm].draw_background(xs, xe - 1);
   	    xs = xe;
         }
 	apply_change(&background_changes, i);
     }
     if (xs <= SCREEN_WIDTH - 1) {
-        int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+	int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+
         video_modes[vm].draw_background(xs, SCREEN_WIDTH - 1);
     }
 
@@ -966,14 +1061,16 @@ inline static void handle_visible_line_with_changes(void)
 	int xe = foreground_changes.actions[i].where;
 
         if (xs < xe) {
-            int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+	    int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+
 	    video_modes[vm].draw_foreground(xs, xe - 1);
 	    xs = xe;
 	}
 	apply_change(&foreground_changes, i);
     }
     if (xs <= SCREEN_TEXTCOLS - 1) {
-        int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+	int vm = draw_idle_state ? SCREEN_IDLE_MODE : video_mode;
+
         video_modes[vm].draw_foreground(xs, SCREEN_TEXTCOLS - 1);
     }
 
@@ -981,11 +1078,12 @@ inline static void handle_visible_line_with_changes(void)
     draw_sprites();
 #endif
 
-    /* Draw left border. */
+    /* Draw left border.  */
     xstop = display_xstart - 1;
     if (!open_left_border) {
-	for (xs = i = 0; 
-	     i < border_changes.count && border_changes.actions[i].where <= xstop; 
+	for (xs = i = 0;
+	     (i < border_changes.count
+	      && border_changes.actions[i].where <= xstop);
 	     i++) {
 	    int xe = border_changes.actions[i].where;
 
@@ -1005,11 +1103,11 @@ inline static void handle_visible_line_with_changes(void)
 	    apply_change(&border_changes, i);
     }
 
-    /* Draw right border. */
+    /* Draw right border.  */
     if (!open_right_border) {
-	for (;  
-	     (i < border_changes.count 
-	      && border_changes.actions[i].where <= display_xstop); 
+	for (;
+	     (i < border_changes.count
+	      && border_changes.actions[i].where <= display_xstop);
 	     i++)
 	    apply_change(&border_changes, i);
 	for (xs = display_xstop; i < border_changes.count; i++) {
@@ -1032,11 +1130,11 @@ inline static void handle_visible_line_with_changes(void)
     background_changes.count =  0;
     border_changes.count = 0;
 
-    /* Do not cache this line. */
+    /* Do not cache this line.  */
     cache[rasterline].is_dirty = 1;
-    
+
     add_line(rasterline, 0, SCREEN_WIDTH - 1);
-    if (pixel_height == 2 && app_resources.doubleScan)
+    if (pixel_height == 2 && double_scan_enabled)
         vid_memcpy(FRAME_BUFFER_LINE_START(frame_buffer, 2*rasterline + 1)
 		   + 2 * SCREEN_MAX_SPRITE_WIDTH,
                    frame_buffer_ptr, pixel_width * SCREEN_WIDTH);
@@ -1048,8 +1146,10 @@ inline static void handle_visible_line(void)
 {
     if (have_changes_on_this_line)
 	handle_visible_line_with_changes();
-    else if (app_resources.videoCache
-	     && !open_left_border && !open_right_border) /* FIXME: shortcut! */
+    else if (!CANVAS_USES_TRIPLE_BUFFERING(canvas)
+             && video_cache_enabled
+	     && !open_left_border
+	     && !open_right_border) /* FIXME: shortcut! */
 	handle_visible_line_with_cache();
     else
 	handle_visible_line_without_cache();
@@ -1057,7 +1157,10 @@ inline static void handle_visible_line(void)
 #if !defined (__VIC_II__) && !defined (__VDC__)
     if (++ycounter >= SCREEN_CHARHEIGHT) {
 	ycounter = 0;
-	memptr += SCREEN_TEXTCOLS;
+	memptr += memptr_inc;
+#ifdef __CRTC__
+	memptr &= addr_mask;
+#endif
     }
 #endif
 }
@@ -1071,6 +1174,9 @@ inline static void handle_end_of_frame(void)
 #ifndef __VIC_II__
     ycounter = 0;
 #endif
+#ifdef __CRTC__
+    memptr = scraddr & addr_mask;
+#endif
 
     if (!skip_next_frame) {
 	if (dont_cache)
@@ -1079,19 +1185,19 @@ inline static void handle_end_of_frame(void)
 	    refresh_changed();
     }
     skip_next_frame = do_vsync(skip_next_frame);
-    
+
 #if defined(__MSDOS__) && !defined(VIC20)
     if (window_width == SCREEN_XPIX && window_height == SCREEN_YPIX)
 	canvas_set_border_color(canvas, border_color);
 #endif
 }
 
-/* Emulate one raster line. */
+/* Emulate one raster line.  */
 inline static void emulate_line(void)
 {
     oldclk += CYCLES_PER_LINE;
 
-    /* Emulate the vertical blank flip-flops. (Well, sort of) */
+    /* Emulate the vertical blank flip-flops.  (Well, sort of.)  */
     if (rasterline == display_ystart && !blank)
 	blank_enabled = 0;
     else if (rasterline == display_ystop)
@@ -1099,18 +1205,22 @@ inline static void emulate_line(void)
 
     if (rasterline >= SCREEN_FIRST_DISPLAYED_LINE
 	&& rasterline <= SCREEN_LAST_DISPLAYED_LINE) {
-	if (!skip_next_frame && !asleep && rasterline >= window_first_line
-	    && rasterline <= window_last_line) {
-	    if (blank_enabled && !open_left_border)
+	if (!skip_next_frame && !asleep
+            && (rasterline >= window_first_line
+                && rasterline <= window_last_line)) {
+	    if ((blank_this_line || blank_enabled) && !open_left_border)
 		handle_blank_line();
 	    else
 		handle_visible_line();
-
 	    if (++num_cached_lines == (window_last_line - window_first_line)) {
 		dont_cache = 0;
 		num_cached_lines = 0;
 	    }
 	} else {
+#ifdef __VIC_II__
+	    if (!skip_next_frame)
+		update_sprite_collisions();
+#endif
 	    if (have_changes_on_this_line) {
 	        apply_all_changes(&background_changes);
 	        apply_all_changes(&foreground_changes);
@@ -1120,11 +1230,11 @@ inline static void emulate_line(void)
 	}
 
 #ifdef __VIC_II__
-	if (!idle_state) 
+	if (!idle_state)
 	    mem_counter = (mem_counter + mem_counter_inc) & 0x3ff;
 	mem_counter_inc = SCREEN_TEXTCOLS;
 	/* `ycounter' makes the chip go to idle state when it reaches the
-	   maximum value. */
+	   maximum value.  */
 	if (ycounter == 7) {
 	    idle_state = 1;
 	    memptr = mem_counter;
@@ -1140,7 +1250,7 @@ inline static void emulate_line(void)
 	draw_idle_state = idle_state;
 	bad_line = 0;
 #endif
-	
+
 	rasterline++;
 	frame_buffer_ptr = FRAME_BUFFER_LINE_START(frame_buffer,
 						   rasterline * pixel_height);
@@ -1150,8 +1260,14 @@ inline static void emulate_line(void)
  	if (rasterline == 0x30)
 	    allow_bad_lines = !blank;
 #endif
-	
+
     } else {
+
+#ifdef __VIC_II__
+	if (!skip_next_frame)
+	    update_sprite_collisions();
+#endif
+
         if (have_changes_on_this_line) {
             apply_all_changes(&background_changes);
             apply_all_changes(&foreground_changes);
@@ -1166,16 +1282,13 @@ inline static void emulate_line(void)
 						    rasterline * pixel_height);
 	    frame_buffer_ptr += 2 * SCREEN_MAX_SPRITE_WIDTH;
 	}
-#ifndef __VIC_II__
-	if (rasterline == 0)
-	    memptr = ycounter = 0;
-#endif
+
     }
 
     apply_all_changes(&next_line_changes);
 
 #ifdef __VIC_II__
-    /* Handle open borders. FIXME: This is not cached! */
+    /* Handle open borders.  */
     open_left_border = open_right_border;
     open_right_border = 0;
 #endif
@@ -1186,9 +1299,10 @@ inline static void emulate_line(void)
 
     memory_fetch_done = 0;
 
+    blank_this_line = 0;
 }
 
-/* Disable all the caching for the next frame. */
+/* Disable all the caching for the next frame.  */
 inline static void force_repaint(void)
 {
     dont_cache = 1;
