@@ -1,15 +1,26 @@
 /*
- * $Id: warn.c,v 1.1 1997/05/22 21:05:13 tvr Exp $
+ * warn.c - General-purpose warning message handler.
+ *
+ * Written by
+ *  Teemu Rantanen (tvr@cs.hut.fi)
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
  *
- * Written by
- *   Teemu Rantanen (tvr@cs.hut.fi)
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- * $Log: warn.c,v $
- * Revision 1.1  1997/05/22 21:05:13  tvr
- * Initial revision
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ *  02111-1307  USA.
  *
  */
 
@@ -18,7 +29,9 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "vice.h"
 #include "warn.h"
+#include "utils.h"
 
 struct warn_s
 {
@@ -30,11 +43,12 @@ struct warn_s
 
 static warn_t *warnlist = NULL;
 
-warn_t *warn_init(char *name, int nrwarnings)
+warn_t *warn_init(const char *name, int nrwarnings)
 {
     warn_t		*p;
+
     p = malloc(sizeof(*p));
-    p->name = name;
+    p->name = stralloc(name);
     p->pnext = warnlist;
     warnlist = p;
     p->nrwarn = nrwarnings;
@@ -81,3 +95,11 @@ void warn_reset(warn_t *pwarn)
     for (p = warnlist; p; p = p->pnext)
 	warn_reset(p);
 }
+
+void warn_free(warn_t *pwarn)
+{
+    free(pwarn->name);
+    if (pwarn->pwarn)
+	free(pwarn->pwarn);
+    free(pwarn);
+}
